
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

package com.iver.cit.gvsig.fmap.operation.strategies;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.List;

import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.PrintQuality;

import org.cresques.cts.ICoordTrans;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.values.NullValue;
import com.hardcode.gdbms.engine.values.NumericValue;
import com.hardcode.gdbms.engine.values.StringValue;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.FPoint2D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FLabel;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.Annotation_Layer;
import com.iver.cit.gvsig.fmap.layers.Annotation_Mapping;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.operations.strategies.DefaultStrategy;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.fmap.rendering.Annotation_Legend;
import com.iver.utiles.swing.threads.Cancellable;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.IntersectionMatrix;


/**
 * Esta clase se encargar de dibujar de la forma ms eficiente los temas de
 * anotaciones.
 *
 * @author Vicente Caballero Navarro
 */
public class Annotation_Strategy extends DefaultStrategy {
    private FSymbol symbolPoint = new FSymbol(FShape.POINT, Color.black);
    private Annotation_Layer capa;
    private static AffineTransform ati=new AffineTransform();
    /**
     * Crea un nuevo AnotationStrategy.
     *
     * @param layer
     */
    public Annotation_Strategy(FLayer layer) {
        super(layer);
        capa = (Annotation_Layer) getCapa();
        symbolPoint.setSize(5);
    }

      public void draw(BufferedImage image, Graphics2D g, ViewPort viewPort,
        Cancellable cancel) {
    	  drawAnnotations(image,g,viewPort,cancel,null);
    }
      private void drawAnnotations(BufferedImage image, Graphics2D g, ViewPort viewPort,
    	        Cancellable cancel,PrintRequestAttributeSet properties) {
    	        BufferedImage bi=new BufferedImage(image.getWidth(),image.getHeight(),BufferedImage.TYPE_INT_ARGB);
    	        Graphics2D gBi=(Graphics2D)bi.getGraphics();
    	        Point2D offset=viewPort.getOffset();
    	        gBi.translate(-offset.getX(),-offset.getY());
    	        Annotation_Legend l = (Annotation_Legend) capa.getLegend();
    	        FSymbol sym = (FSymbol) l.getDefaultSymbol();

    	        try {
    	            ReadableVectorial source = capa.getSource();
    	           // limit the labeling to the visible extent
    	            FBitSet bs = capa.queryByRect(viewPort.getAdjustedExtent());

    	            SelectableDataSource recordSet = source.getRecordset();
    	            recordSet.start();
    	            FBitSet bitSet = recordSet.getSelection();
    	            Annotation_Mapping mapping = ((Annotation_Layer) capa).getAnnotatonMapping();
    	            int idHeightField = mapping.getColumnHeight();
    	            int idFontName = mapping.getColumnTypeFont();
    	            int idFontStyle = mapping.getColumnStyleFont();
    	            int idRotationField = mapping.getColumnRotate();
    	            int idFontColor = mapping.getColumnColor();
    	            int idTextField = mapping.getColumnText();

    	            double rotation = 0D;
    	            float size = sym.getFont().getSize()* (float)FConstant.FONT_HEIGHT_SCALE_FACTOR;

    	            String fontName = "Dialog";
    	            int fontStyle = sym.getFont().getStyle();
    	            int fontColor = sym.getFontColor().getRGB();
    	            long t1 = System.currentTimeMillis();

    	            for (int i = bs.nextSetBit(0); i >= 0; i = bs.nextSetBit(i + 1)) {
    	                if (cancel.isCanceled()) {
    	                    break;
    	                }

    	                Value[] vv = recordSet.getRow(i);

    	                if (idHeightField != -1) {
    	                    // text size is defined in the table
    	                    try {
    	                        size = (float)(((NumericValue) vv[idHeightField]).doubleValue() * FConstant.FONT_HEIGHT_SCALE_FACTOR);
    	                    } catch (ClassCastException ccEx) {
    	                        if (!NullValue.class.equals(
    	                                    vv[idHeightField].getClass())) {
    	                        }
    	                        continue;
    	                    }
    	                }

    	                if (idFontName != -1) {
    	                    fontName = ((StringValue) vv[idFontName]).toString();
    	                }

    	                if (idFontStyle != -1) {
    	                    fontStyle = ((NumericValue) vv[idFontStyle]).intValue();
    	                }

    	                if (idRotationField != -1) {
    	                    // text rotation is defined in the table
    	                    rotation = ((NumericValue) vv[idRotationField]).doubleValue();
    	                    sym.setRotation((int)rotation);
    	                }

    	                if (idFontColor != -1) {
    	                    // text rotation is defined in the table
    	                    fontColor = ((NumericValue) vv[idFontColor]).intValue();
    	                    sym.setFontColor(new Color(fontColor));
    	                }

    	                if (bitSet.get(i)) {
    	                    sym = (FSymbol) sym.getSymbolForSelection();
    	                }

    	                sym.setFont(new Font(fontName, fontStyle, (int) size));
    	                source.start();
    	                IGeometry geom = source.getShape(i);
    	                source.stop();
    	                ICoordTrans ct = capa.getCoordTrans();

	                     boolean bMustClone = false;

	                     if (geom == null) {
	                         return;
	                     }

	                     if (ct != null) {
	                         if (bMustClone) {
	                             geom = geom.cloneGeometry();
	                         }

	                         geom.reProject(ct);
	                     }
    	                geom.transform(viewPort.getAffineTransform());
    	                Shape shape=geom.getInternalShape();
    	                FPoint2D fpPixels=null;
    	                if (!(shape instanceof FPoint2D)) {
    	                	Rectangle2D rP=shape.getBounds2D();
    	                	fpPixels=new FPoint2D(rP.getX(),rP.getMaxY());
    	                }else {
    	                	fpPixels=(FPoint2D)shape;
    	                }
    	                //Si el tamao de la fuente est en unidades de mapa.
    	                if (!sym.isFontSizeInPixels()){
    	                	boolean draw=false;
    	                	Rectangle2D r=capa.getTextWrappingGeometryInPixels(size,vv[idTextField].toString(),rotation,fontName, fontStyle,i,viewPort,geom).getBounds2D();
    		                Rectangle2D rPixels=new Rectangle2D.Double(r.getX(),r.getY()+r.getHeight(),r.getWidth(),r.getHeight());

    		                Point2D p=new Point2D.Double(fpPixels.getX(),fpPixels.getY());

    		                if (!capa.isEditing() && l.isAvoidOverLapping()){
    		                	p=getPoint(bi,rPixels,l.isDelOverLapping());
    		                	if (p!=null)
    		                		draw=true;
    		                }else{
    		                	if (!capa.isEditing() && l.isDelOverLapping()){
    		                		if (isOverlapping(bi,rPixels)){
    		                			draw=true;
    		                		}
    		                	}else{
    		                		draw=true;
    		                	}
    		                }
    		                if (draw){
    		                    if (sym.isShapeVisible()) {
    		                        symbolPoint.draw(gBi,
    		                               ati,
    		                               fpPixels);
    		                    }
//    		                    g.drawRect((int)rPixels.getX(),(int)rPixels.getY(),(int)rPixels.getWidth(),(int)rPixels.getHeight());
//    		                    FGraphicUtilities.DrawAnnotation(gBi,
//    		                            ati, sym, aux[0], metrics, false);
    		                   	DrawAnnotation(gBi,sym, p, vv[idTextField].toString(),viewPort,properties);
    		                }
    	                }else{

    	                	// Si el tamao de la fuente est en pixels.
    	                	boolean draw=false;

    	                	Rectangle2D r=capa.getTextWrappingGeometryInPixels(size,vv[idTextField].toString(),rotation,fontName,fontStyle,i,viewPort,geom).getBounds2D();
//    	                	g.draw(capa.getTextWrappingGeometryInPixels(size,vv[idTextField].toString(),rotation,fontName,fontStyle,i,viewPort,geom));
    		                Rectangle2D rPixels= new Rectangle2D.Double(r.getX(),r.getY()+r.getHeight(),r.getWidth(),r.getHeight());

    		                Point2D pSearch = new Point2D.Double(fpPixels.getX(),fpPixels.getY());

    		                if (!capa.isEditing() && l.isAvoidOverLapping()){
    		                	pSearch =getPoint(bi,rPixels,l.isDelOverLapping());
    		                	if (pSearch !=null){
    		                		draw=true;
    		                	}
    		                }else{
    		                	if (!capa.isEditing() && l.isDelOverLapping()){
    		                		if (isOverlapping(bi,rPixels)){
    		                			draw=true;
    		                		}
    		                	}else{
    		                		draw=true;
    		                	}
    		                }
    		                if (draw){
    		                    if (sym.isShapeVisible()) {
    		                        symbolPoint.draw(gBi,
    		                               ati,
    		                               fpPixels);
    		                    }
//    		                    g.drawRect((int)pSearch.getX(),(int)(pSearch.getY()-rPixels.getHeight()),(int)rPixels.getWidth(),(int)rPixels.getHeight());
//    		                    FGraphicUtilities.DrawAnnotation(gBi,
//    		                            ati, sym, aux[0], metrics, false);
    		                   	DrawAnnotation(gBi,sym, pSearch , vv[idTextField].toString(),viewPort,properties);
    		                }
    	                }
    	                //Al dibujar esta imagen sobre el graphics la trasparencia no puede existir.
    	                if (i%1000==0){
    	                	 g.drawImage(bi,0,0,null);
    	                }
    	            }
    	            gBi.translate(offset.getX(),offset.getY());
    	            g.drawImage(bi,(int)offset.getX(),(int)offset.getY(),null);
    	            System.err.println(System.currentTimeMillis()-t1+"millis");
    	            recordSet.stop();
    	        } catch (Exception e) {
    	            NotificationManager.addError(e);
    	        }
    	    }
    private Point2D getPoint(BufferedImage bi, Rectangle2D rec, boolean b) {
		Rectangle2D r=new Rectangle2D.Double(rec.getX(),rec.getY()-rec.getHeight(),rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		r.setFrame(rec.getX(),rec.getY(),rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		r.setFrame(rec.getX()-rec.getWidth(),rec.getY()-rec.getHeight(),rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		r.setFrame(rec.getX()-rec.getWidth(),rec.getY(),rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		r.setFrame(rec.getX()-rec.getWidth()/2,rec.getY(),rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		r.setFrame(rec.getX()-rec.getWidth()/2,rec.getY()-rec.getHeight(),rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		r.setFrame(rec.getX(),rec.getY()+rec.getHeight()/2,rec.getWidth(),rec.getHeight());
		if (isOverlapping(bi,r))
			return new Point2D.Double(r.getX(),r.getY()+rec.getHeight());

		if (!b)
			return new Point2D.Double(rec.getX(),rec.getY());
		else
			return null;
	}

	private boolean isOverlapping(BufferedImage bi, Rectangle2D rPixels) {
        for (int i=(int)rPixels.getX();i<rPixels.getMaxX();i++){
            for (int j=(int)rPixels.getY();j<rPixels.getMaxY();j++){
                if (i<0 || j<0 || bi.getWidth()<i+1 || bi.getHeight()<j+1)
                    continue;
                if (bi.getRGB(i,j)!=0){
                    return false;
                }
            }
        }
        return true;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByShape(com.iver.cit.gvsig.fmap.core.IGeometry,
     *      int)
     */
    public FBitSet queryByShape(IGeometry g, int relationship)
        throws DriverException, VisitException {
        // Si hay un ndice espacial, lo usamos para hacer el query.
        FLyrVect lyr = (FLyrVect) capa;

        // if (lyr.getSpatialIndex() == null)
        if (lyr.getISpatialIndex() == null) {
            return super.queryByShape(g, relationship);
        }

        long t1 = System.currentTimeMillis();
        ReadableVectorial va = lyr.getSource();
        ICoordTrans ct = lyr.getCoordTrans();
        Rectangle2D bounds = g.getBounds2D();

        // Coordinate c1 = new Coordinate(bounds.getMinX(), bounds.getMinY());
        // Coordinate c2 = new Coordinate(bounds.getMaxX(), bounds.getMaxY());
        // Envelope env = new Envelope(c1, c2);
        // List lstRecs = lyr.getSpatialIndex().query(env);
        List lstRecs = lyr.getISpatialIndex().query(bounds);
        Integer idRec;
        FBitSet bitset = new FBitSet();
        Geometry jtsShape = g.toJTSGeometry();
        IntersectionMatrix m;
        int index;

        try {
            va.start();

            // Annotation_Legend aLegend=(Annotation_Legend)capa.getLegend();
            for (int i = 0; i < lstRecs.size(); i++) {
                idRec = (Integer) lstRecs.get(i);
                index = idRec.intValue();

                IGeometry geom = va.getShape(index);

                // FSymbol symbol=(FSymbol)aLegend.getSymbol(index);
                // IGeometry geom=aLegend.getTextWrappingGeometry(symbol,index);
                // IGeometry
                // geom=getGeometry(((Annotation_Layer)capa).getLabel(index).getBoundBox());
                if (ct != null) {
                    geom.reProject(ct);
                }

                Geometry jtsGeom = geom.toJTSGeometry();

                switch (relationship) {
                case CONTAINS:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isContains()) {
                        bitset.set(index, true);
                    }

                    break;

                case CROSSES:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isCrosses(jtsGeom.getDimension(),
                                jtsShape.getDimension())) {
                        bitset.set(index, true);
                    }

                    break;

                case DISJOINT:

                    // TODO: CREO QUE EL DISJOINT NO SE PUEDE METER AQUI
                    m = jtsShape.relate(jtsGeom);

                    if (m.isDisjoint()) {
                        bitset.set(index, true);
                    }

                    break;

                case EQUALS:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isEquals(jtsGeom.getDimension(),
                                jtsShape.getDimension())) {
                        bitset.set(index, true);
                    }

                    break;

                case INTERSECTS:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isIntersects()) {
                        bitset.set(index, true);
                    }

                    break;

                case OVERLAPS:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isOverlaps(jtsGeom.getDimension(),
                                jtsShape.getDimension())) {
                        bitset.set(index, true);
                    }

                    break;

                case TOUCHES:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isTouches(jtsGeom.getDimension(),
                                jtsShape.getDimension())) {
                        bitset.set(index, true);
                    }

                    break;

                case WITHIN:
                    m = jtsShape.relate(jtsGeom);

                    if (m.isWithin()) {
                        bitset.set(index, true);
                    }

                    break;
                }
            }

            va.stop();
        } catch (DriverIOException e) {
        	throw new DriverException(e);
        }

        long t2 = System.currentTimeMillis();

        // logger.debug("queryByShape optimizado sobre la capa " + lyr.getName()
        // + ". " + (t2-t1) + " mseg.");
        return bitset;
    }

    /**
     * DOCUMENT ME!
     *
     * @param rect DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws DriverException DOCUMENT ME!
     */
    public FBitSet queryByRect(Rectangle2D rect) throws DriverException {
        // Si hay un ndice espacial, lo usamos para hacer el query.
        Annotation_Layer lyr = (Annotation_Layer) capa;
        ReadableVectorial va = lyr.getSource();
        ICoordTrans ct = lyr.getCoordTrans();
        Rectangle2D bounds = rect;
        // if (lyr.getSpatialIndex() == null)
        if (lyr.getISpatialIndex() == null) {
            return super.queryByRect(rect);
        }
        // Coordinate c1 = new Coordinate(bounds.getMinX(), bounds.getMinY());
        // Coordinate c2 = new Coordinate(bounds.getMaxX(), bounds.getMaxY());
        // Envelope env = new Envelope(c1, c2);
        //
        // List lstRecs = lyr.getSpatialIndex().query(env);
        // azabala
        List lstRecs = lyr.getISpatialIndex().query(bounds);
        Integer idRec;
        FBitSet bitset = new FBitSet();
        int index;

        try {
            va.start();

            DriverAttributes attr = va.getDriverAttributes();
            boolean bMustClone = false;

            if (attr != null) {
                if (attr.isLoadedInMemory()) {
                    bMustClone = attr.isLoadedInMemory();
                }
            }
            ViewPort vp=getCapa().getMapContext().getViewPort();
            SelectableDataSource sds=va.getRecordset();
            // Annotation_Legend aLegend=(Annotation_Legend)capa.getLegend();
            for (int i = 0; i < lstRecs.size(); i++) {
                idRec = (Integer) lstRecs.get(i);
                index = idRec.intValue();

                Annotation_Mapping mapping = ((Annotation_Layer) capa).getAnnotatonMapping();
                NumericValue vRotation = (NumericValue) sds.getFieldValue(index,
                        mapping.getColumnRotate());
                NumericValue vHeight = (NumericValue) sds.getFieldValue(index,
                        mapping.getColumnHeight());
                NumericValue vStyle = (NumericValue) sds.getFieldValue(index,mapping.getColumnStyleFont());
                StringValue vType = (StringValue) sds.getFieldValue(index,mapping.getColumnTypeFont());
                Value vText = sds.getFieldValue(index,
                        mapping.getColumnText());
                va.start();
                IGeometry geom = va.getShape(i);
                va.stop();

                if (ct != null) {
                    if (bMustClone) {
                        geom = geom.cloneGeometry();
                    }

                    geom.reProject(ct);
                }
                geom.transform(vp.getAffineTransform());
                IGeometry geom1 = ((Annotation_Layer) capa).getTextWrappingGeometryInPixels(vHeight.floatValue(),
                        vText.toString(), vRotation.doubleValue(),vType.getValue(),vStyle.intValue(), index, vp,geom);
                geom1.transform(vp.getAffineTransform().createInverse());
                if (geom1.intersects(rect)) {
                    bitset.set(index, true);
                }
            }

            va.stop();
        } catch (DriverIOException e) {
            throw new DriverException(e);
        } catch (DriverLoadException e) {
        	throw new DriverException(e);
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
        	throw new DriverException(e);
        } catch (NoninvertibleTransformException e) {
        	throw new DriverException(e);
		}

        return bitset;
    }

    /*
     * (non-Javadoc)
     *
     * @see com.iver.cit.gvsig.fmap.operations.strategies.Strategy#queryByPoint(java.awt.geom.Point2D,
     *      double)
     */
    public FBitSet queryByPoint(Point2D p, double tolerance)
        throws DriverException {
        // TODO: OJO!!!!. Est implementado como un rectangulo.
        // Lo correcto debera ser calculando las distancias reales
        // es decir, con un crculo.
        Rectangle2D recPoint = new Rectangle2D.Double(p.getX() -
                (tolerance / 2), p.getY() - (tolerance / 2), tolerance,
                tolerance);

        return queryByRect(recPoint);
    }

    public void print(Graphics2D g, ViewPort viewPort, Cancellable cancel,
        PrintRequestAttributeSet properties) throws DriverException {
        capa.beforePrinting(properties);
        drawAnnotations(null,g, viewPort, cancel, properties);
        capa.afterPrinting();
    }
    private static void DrawAnnotation(Graphics2D g2,
    		FSymbol theSymbol,Point2D pAux, String text,ViewPort viewPort,PrintRequestAttributeSet properties) {
    		float x;
    		float y;
    		// Las etiquetas que pongamos a nulo ser porque no la queremos dibujar.
    		// til para cuando queramos eliminar duplicados.
    		if (text == null) {
    			return;
    		}
    		int size=theSymbol.getFont().getSize();
    		double resolutionPrinting=0;
    		if (!theSymbol.isFontSizeInPixels()){
    			size=viewPort.fromMapDistance(size);
    		}
    		if (properties!=null){
    			PrintQuality resolution=(PrintQuality)properties.get(PrintQuality.class);
    			if (resolution.equals(PrintQuality.NORMAL)){
    				resolutionPrinting=300/72;
    			}else if (resolution.equals(PrintQuality.HIGH)){
    				resolutionPrinting=600/72;
    			}else if (resolution.equals(PrintQuality.DRAFT)){
    				resolutionPrinting=72/72;
    			}
    			size=(int)(size*resolutionPrinting);
    		}
    		Font font=new Font(theSymbol.getFont().getName(),theSymbol.getFont().getStyle(),size);
   			g2.setFont(font);
    		g2.setColor(theSymbol.getFontColor());
    		AffineTransform ant = g2.getTransform();

    		x = (float) pAux.getX();
    		y = (float) pAux.getY();

    		AffineTransform Tx = (AffineTransform) ant.clone();
    		Tx.translate(x, y); // S3: final translation
    		Tx.rotate(Math.toRadians(-theSymbol.getRotation())); // S2: rotate around anchor
    		g2.setTransform(Tx);
			g2.drawString(text, 0, 0);
    		g2.setTransform(ant);
    	}

}
