/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.rasterTools.saveRaster;

import org.gvsig.rasterTools.saveRaster.ui.listener.SaveRasterListener;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.MouseMovementBehavior;
import com.iver.cit.gvsig.fmap.tools.Behavior.RectangleBehavior;
import com.iver.cit.gvsig.project.documents.view.IProjectView;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.cit.gvsig.project.documents.view.toolListeners.StatusBarListener;


/**
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class SaveRasterModule extends Extension {
    /* (non-Javadoc)
     * @see com.iver.andami.plugins.Extension#inicializar()
     */
    public void initialize() {
    }

    /**
     * Carga el listener de salvar a raster en el MapControl.
     */
    private void loadListener(MapControl m_MapControl){
        //Si no se ha cargado el listener an lo cargamos.
        if(m_MapControl.getNamesMapTools().get("saveRaster") == null){
	        //Listener de eventos de movimiento que pone las coordenadas del ratn en la barra de estado
	        StatusBarListener sbl = new StatusBarListener(m_MapControl);

	        //Salvar a Raster
	        SaveRasterListener srl = new SaveRasterListener(m_MapControl);
	        m_MapControl.addMapTool("saveRaster", new Behavior[]{
	        				new RectangleBehavior(srl), new MouseMovementBehavior(sbl)});
        }
    }


    /**
     * Cargamos el listener y ejecutamos la herramienta de salvar a Raster.
     */
    public void execute(String actionCommand) {
    	View theView = (View) PluginServices.getMDIManager().getActiveWindow();
    	MapControl m_MapControl = theView.getMapControl();
    	loadListener(m_MapControl);
    	m_MapControl.setTool("saveRaster");
    }

    /* (non-Javadoc)
     * @see com.iver.andami.plugins.Extension#isEnabled()
     */
    public boolean isEnabled() {
        return true;
    }

    /**
     * Mostramos el control si hay alguna capa cargada.
     */
    public boolean isVisible() {
        com.iver.andami.ui.mdiManager.IWindow f = PluginServices.getMDIManager()
                                                             .getActiveWindow();
        if (f == null)
            return false;

        if (f instanceof View) {
		    View vista = (View) f;
		    IProjectView model = vista.getModel();
		    MapContext mapa = model.getMapContext();
            if (mapa.getLayers().getLayersCount() > 0)
                return true;
		}

		return false;
    }
}
