/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package com.iver.cit.gvsig.gui.toc;

import java.awt.event.ActionEvent;

import javax.swing.JMenuItem;

import org.cresques.filter.RasterFilter;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;
import org.cresques.px.PxRaster;
import org.cresques.ui.BrightnessContrast.EnhancedBrightnessContrastPanel;
import org.cresques.ui.filter.FilterRasterDialogPanel;
import org.cresques.ui.filter.RasterTransparencyPanel;
import org.cresques.ui.raster.BandSetupPanel;
import org.cresques.ui.raster.InfoPanel;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrWCS;
import com.iver.cit.gvsig.gui.dialog.WCSRasterPropsDialog;
import com.iver.cit.gvsig.project.documents.view.toc.TocMenuEntry;
import com.iver.cit.gvsig.project.documents.view.toc.gui.FPopupMenu;

/**
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class WCSPropsRasterTocMenuEntry  extends TocMenuEntry {
	private JMenuItem propsMenuItem;
	FLayer lyr = null;
	private WCSRasterPropsDialog	propsDialog = null;
	private BandSetupPanel 			bandSetup = null;
	
	public void initialize(FPopupMenu m) {
		super.initialize(m);
		
		if (isTocItemBranch()) {
			lyr = getNodeLayer();
    		// Opcciones para capas WCS
    		if ((lyr instanceof FLyrWCS)) {
    			propsMenuItem = new JMenuItem(PluginServices.getText(this, "propiedades_raster"));
    			getMenu().add( propsMenuItem );
    			propsMenuItem.setFont(FPopupMenu.theFont);
    			getMenu().setEnabled(true);
    			//getMenu().addSeparator();
    	        //Cambio color
    			propsMenuItem.addActionListener(this);   			
     		}
		}
	}
	/* (non-Javadoc)
	 * @see com.iver.cit.gvsig.gui.toc.TocMenuEntry#execute(com.iver.cit.gvsig.gui.toc.ITocItem)
	 */
	public void actionPerformed(ActionEvent e) {
       	lyr = getNodeLayer();
        
		if(lyr instanceof FLyrWCS && ((FLyrWCS) lyr).getPxRaster()!=null){
			RasterFilterStackManager 	stackManager = null;
			
			FLyrWCS layer = (FLyrWCS) lyr;

			stackManager = new RasterFilterStackManager(layer.getFilterStack());
			propsDialog = new WCSRasterPropsDialog(layer, stackManager.getTransparencyRGB());
			
			int alpha = layer.getPxRaster().getAlpha();
			
			FilterRasterDialogPanel filterPanel = ((FilterRasterDialogPanel)propsDialog.getContentPane());
			
			bandSetup = (BandSetupPanel)filterPanel.getPanelByClassName("BandSetupPanel");
			GeoRasterFile[] files = layer.getPxRaster().getFiles();
			bandSetup.addFiles(files);
			for(int i=0; i< files.length;i++)
				propsDialog.addNumBands(files[i].getBandCount());
				
			PxRaster px = layer.getPxRaster();
			int posR = 0, posG = 0, posB = 0;
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.RED_BAND);i++)
				posR += files[i].getBandCount();
			posR += px.getBand(GeoRasterFile.RED_BAND);
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.GREEN_BAND);i++)
				posG += files[i].getBandCount();
			posG += px.getBand(GeoRasterFile.GREEN_BAND);
			
			for(int i=0;i<px.getPosFile(GeoRasterFile.BLUE_BAND);i++)
				posB += files[i].getBandCount();
			posB += px.getBand(GeoRasterFile.BLUE_BAND);
			
					
			bandSetup.assignBand(posR, GeoRasterFile.RED_BAND);
			bandSetup.assignBand(posG, GeoRasterFile.GREEN_BAND);
			bandSetup.assignBand(posB, GeoRasterFile.BLUE_BAND);	

			InfoPanel pInfo = (InfoPanel)this.propsDialog.getPanelByClassName("InfoPanel");
			pInfo.setBands(posR, posG, posB);
			pInfo.addFiles(files);	
			
			//Asignacin del alpha actual de la imagen al dialogo
			RasterTransparencyPanel rasterTrans = (RasterTransparencyPanel)filterPanel.getPanelByClassName("RasterTransparencyPanel");
			rasterTrans.setOpacity(alpha);
							
			propsDialog.setRasterFilterStackManager(stackManager);							
			propsDialog.readStat();
			
			actionPerformed(stackManager, propsDialog, null);
			
			PluginServices.getMDIManager().addWindow(propsDialog);       	
		}
	}
	
	/**
	 * Modificacin del estado de los controles del panel de brillo y contraste
	 */
	public void actionPerformed(RasterFilterStackManager stackManager, WCSRasterPropsDialog propsDialog, FLyrRaster fLayer) {
		EnhancedBrightnessContrastPanel bcPanel = (EnhancedBrightnessContrastPanel)((FilterRasterDialogPanel)propsDialog.getContentPane()).getPanelByClassName("EnhancedBrightnessContrastPanel"); 
		//bcPanel.setPropertiesDialog(propsDialog);
		
		if(stackManager.isActive(stackManager.getTypeFilter("brightness"))){
			bcPanel.getCBrightC().setSelected(true);
			bcPanel.setBCControlEnabled(true);
			
			RasterFilter bright = stackManager.getFilter("brightness");
			
			if(bright.getParam("incrBrillo") != null){
				int incr = ((Integer)bright.getParam("incrBrillo")).intValue();
				bcPanel.getLabelSliderText().setSliderValue(incr);
				bcPanel.getLabelSliderText().setTextValue(String.valueOf(incr));
			}
		}
		
		if(stackManager.isActive(stackManager.getTypeFilter("contrast"))){
			bcPanel.getCBrightC().setSelected(true);
			bcPanel.setBCControlEnabled(true);
			
			RasterFilter cont = stackManager.getFilter("contrast");
			
			if(cont.getParam("incrContraste") != null){
				int incr = ((Integer)cont.getParam("incrContraste")).intValue();
				bcPanel.getLabelSliderText1().setSliderValue(incr);
				bcPanel.getLabelSliderText1().setTextValue(String.valueOf(incr));
			}
		}
		
		if(stackManager.isActive(stackManager.getTypeFilter("enhanced"))){
			bcPanel.getCEnhanced().setSelected(true);
			bcPanel.setEControlEnabled(true);
			
			RasterFilter enhan = stackManager.getFilter("enhanced");
			RasterFilter tail = stackManager.getFilter("tail");
			
			
				boolean rem = ((Boolean)enhan.getParam("remove")).booleanValue();
				bcPanel.getJCheckBox().setSelected(rem);
			
			if(stackManager.isActive(stackManager.getTypeFilter("tail"))){
				double percent = (stackManager.getStackStats().tailPercent)*100;
				bcPanel.getCheckSliderText().setTextValue(String.valueOf(percent));
				bcPanel.getCheckSliderText().setSliderValue((int)percent);
				bcPanel.getCheckSliderText().setSelected(true);
				bcPanel.getCheckSliderText().setControlEnabled(true);
				boolean remove = ((Boolean)tail.getParam("remove")).booleanValue();
				if((remove == true) || (rem == true))
					bcPanel.getJCheckBox().setSelected(true);
				else 
					bcPanel.getJCheckBox().setSelected(false);
				
			}
		}
	}
}