/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.bands;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Palette;

/**
 * Filtro de paleta a aplicar sobre una imagen con tipo de dato integer. Para cada elemento
 * del RasterBuf se realiza la conversin de dato a travs de la clase palette que se pasa
 * como parmetro al filtro en la inicializacin.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class PaletteIntFilter extends PaletteFilter {
	private int[] 		px = new int[4];
	
	/**
	 * Constructor
	 *
	 */
	public PaletteIntFilter(){
		super();
	}
	
	
	public void pre(){
		this.raster = (RasterBuf) params.get("raster");
		height = raster.getHeight();
        width = raster.getWidth();
        rasterResult = new RasterBuf(raster.getDataType(),width,height,3,true);
		super.pre();
	}
	
	public void process(int col, int line) {
        raster.getElemInt(line, col, px);
        int rgb;
        if(interpolate)
        	rgb = palette.getInterpolateRGB2(px[0]);
        else
        	rgb = palette.getRGB(px[0]);
		
        int pxOut[] = {(rgb & 0x00ff0000) >> 16,
						(rgb & 0x0000ff00) >> 8, 
						(rgb & 0x000000ff)};
        rasterResult.setElemInt(line, col,pxOut);

	}

	 /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
	public void processSuperSampling(int col, int line) {
		raster.getElemInt(line, col, px);
		int rgb;
        if(interpolate)
        	rgb = palette.getInterpolateRGB2(px[0]);
        else
        	rgb = palette.getRGB(px[0]);
		
        int pxOut[] = {(rgb & 0x00ff0000) >> 16,
				(rgb & 0x0000ff00) >> 8, 
				(rgb & 0x000000ff)};
        for(int j = col; j < width && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < height && i < (line + stepY[contY + 1]); i++)
				rasterResult.setElemInt(i, j,pxOut);		
	}
	
	public void processLine(int y) {
		// TODO Auto-generated method stub
	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_INT;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_INT;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
			if(isExec())
				return (Object) this.rasterResult;
			else
				return (Object) this.raster;
        } else {
            return null;
        }
	}

}
