package org.cresques.filter.convolution;

import org.cresques.filter.RasterFilter;
import org.cresques.io.data.RasterBuf;

public class ConvolutionDoubleFilter extends ConvolutionFilter {

	/**
	 * Constructor sin parmetros. Este es usado principalmente desde controlTypes
	 * ya que se instancia con newInstance y sin parmetros. Poseriormente le aplica
	 * el nombre al filtro con la funcin setFilterName de RasterFilter.
	 */
	public ConvolutionDoubleFilter() {}
	
	/**
	 * Constructor para la asignacin del identificador del filtro
	 * @param fName Cadena que representa el identificador del filtro 
	 */
	public ConvolutionDoubleFilter(String fName) {
		super(fName);
	}
	
	public void process(int col, int line) {
		double[] pxOut = new double[4];
		double[] px = new double[4];
		int ladoVentana = kernel.getLado();
		int semiLado = (ladoVentana-1)>>1;
		
		double ventanaR[][]=new double[ladoVentana][ladoVentana];
		double ventanaG[][]=new double[ladoVentana][ladoVentana];
		double ventanaB[][]=new double[ladoVentana][ladoVentana];
		
		Kernel kernelRGB[]=new Kernel[3];

		raster.getElemDouble(line, col, px); 
		
		if((col-semiLado >= 0) && (line-semiLado >= 0) &&(col+semiLado < width)&&(line+semiLado < height)) 
		{
				// Obtener el vector con la ventanas de muestras (una por componente RGB)
			for (int i=-semiLado;i<=semiLado;i++)
				for(int j=-semiLado;j<=semiLado;j++)
				{
					raster.getElemDouble(line+j, col+i,px);
					ventanaR[i+semiLado][j+semiLado] =px[0];
					ventanaG[i+semiLado][j+semiLado] =px[1];
					ventanaB[i+semiLado][j+semiLado] =px[2];
				}
				kernelRGB[0]=new Kernel(ventanaR);
				kernelRGB[1]=new Kernel(ventanaG);
				kernelRGB[2]=new Kernel(ventanaB);
				
				pxOut[0]= kernel.convolution(kernelRGB[0]);
				pxOut[1]= kernel.convolution(kernelRGB[1]);
				pxOut[2]= kernel.convolution(kernelRGB[2]);
				
				if (umbral>0){
					if(pxOut[0]>=umbral)pxOut[0]=255;
					else pxOut[0]=0;
					if(pxOut[1]>=umbral)pxOut[1]=255;
					else pxOut[1]=0;
					if(pxOut[2]>=umbral)pxOut[2]=255;
					else pxOut[2]=0;
				}
				else{
					if (pxOut[0]<0) pxOut[0]=0;
					else if (pxOut[0]>255) pxOut[0]=255;
					if (pxOut[1]<0) pxOut[1]=0;
					else if (pxOut[1]>255) pxOut[1]=255;
					if (pxOut[2]<0) pxOut[2]=0;
					else if (pxOut[2]>255) pxOut[2]=255;
				}
				
			rasterResult.setElemDouble(line, col,pxOut);
		}
		else rasterResult.setElemDouble(line, col,pxOut);

	}

	public void processSuperSampling(int col, int line) {
		double[] px = new double[4];
		double[] pxOut = new double[4];
		int ladoVentana = kernel.getLado();
		RasterFilter.Kernel kernelRGB[];

		raster.getElemDouble(line, col,px); 
		
		//	Obtener el vector con la ventanas de muestras (una por componente RGB)
		kernelRGB=extractSubRaster(ladoVentana,col,line);
		if(kernelRGB!=null)
		{
				pxOut[0]= kernel.convolution(kernelRGB[0]);
				pxOut[1]= kernel.convolution(kernelRGB[1]);
				pxOut[2]= kernel.convolution(kernelRGB[2]);
				
				if (umbral>0){
					if(pxOut[0]>=umbral)pxOut[0]=255;
					else pxOut[0]=0;
					if(pxOut[1]>=umbral)pxOut[1]=255;
					else pxOut[1]=0;
					if(pxOut[2]>=umbral)pxOut[2]=255;
					else pxOut[2]=0;
				}
				else{
					if (pxOut[0]<0) pxOut[0]=0;
					else if (pxOut[0]>255) pxOut[0]=255;
					if (pxOut[1]<0) pxOut[1]=0;
					else if (pxOut[1]>255) pxOut[1]=255;
					if (pxOut[2]<0) pxOut[2]=0;
					else if (pxOut[2]>255) pxOut[2]=255;
				}
				
			for(int i = col; i < width && i < (col + stepX[contX + 1]); i++)
				for(int j = line; j < height && j < (line + stepY[contY + 1]); j++)	
					rasterResult.setElemDouble(j, i,pxOut);
		}
		else
		for(int i = col; i < rasterResult.getWidth() && i < (col + stepX[contX + 1]); i++)
			for(int j = line; j < rasterResult.getHeight() && j < (line + stepY[contY + 1]); j++)	
				rasterResult.setElemDouble(j, i,px);

	}

	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_DOUBLE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_DOUBLE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.rasterResult;
        } else {
            return null;
        }
	}

	public void pre() {
		exec = true;
		this.raster = (RasterBuf)params.get("raster");
		height = raster.getHeight();
        width = raster.getWidth();
		rasterResult = new RasterBuf(raster.getDataType(),width,height,raster.getBandCount(),true);
		
		super.pre();
	}
	
	/**
	 * @param ladoVentana  Lado del subraster a extraer
	 * @param x  Coordenada x del pixel central
	 * @param y  Coordenada y del pixel central
	 * @return
	 */
	public Kernel[] extractSubRaster(int ladoVentana,int x,int y){
		
		int k,indiceX,indiceY,origenX,origenY;
		double[] 		px = new double[4];
		int semiLado = (ladoVentana-1)>>1;
		int offsetX=0;
		int offsetY=0;
		
		Kernel kernelRGB[] = null;
		
		// Obtener el vector con las ventanas de muestras (una por componente RGB) **************************
		// Me situo en la esquina superior izquierda del "kernel"
		if ((contX+1-semiLado>=0) && (contY+1-semiLado>=0)&& (contX+1+semiLado<stepX.length) && (contY+1+semiLado<stepY.length)){
			
			// Calcular el alcance del kernel cuyo centro es (x,y)
			for (int i=0;i<semiLado;i++){
				offsetX=offsetX+stepX[contX+i+1];
				offsetY=offsetY+stepY[contY+i+1];
			}
			if((x+offsetX<width)&&(y+offsetY<height)){
				
				double ventanaR[][] = new double[ladoVentana][ladoVentana];
				double ventanaG[][] = new double[ladoVentana][ladoVentana];
				double ventanaB[][] = new double[ladoVentana][ladoVentana];
				kernelRGB = new Kernel[3];
				
				origenX=x;
				origenY=y;
				for (int i=0;i<semiLado;i++){
					origenX=origenX-stepX[contX-i];
					origenY=origenY-stepY[contY-i];	
				}
				//Recorro el kernel segn los step
				k=0;
				indiceX=origenX;
				for (int i=-semiLado;i<=semiLado;i++){
					indiceY=origenY;
					for(int j=-semiLado;j<=semiLado;j++){	
						raster.getElemDouble(indiceY,indiceX,px);	
						ventanaR[i+semiLado][j+semiLado] =px[0];
						ventanaG[i+semiLado][j+semiLado] =px[1];
						ventanaB[i+semiLado][j+semiLado] =px[2];
						indiceY=indiceY+stepY[contY+j+1];
						k++;
					}
					indiceX=indiceX+stepX[contX+i+1];
				}
				//**************************************************************************************************
				kernelRGB[0]= new Kernel(ventanaR);
				kernelRGB[1]= new Kernel(ventanaG);
				kernelRGB[2]= new Kernel(ventanaB);
			}
		}
		return kernelRGB;
	}

}
