/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */

package org.cresques.filter.correction;

import java.util.ArrayList;

import org.cresques.filter.IStackManager;
import org.cresques.io.data.RasterBuf;
import org.cresques.filter.RasterFilter;
import org.cresques.filter.RasterFilterStack;
import org.cresques.filter.RasterFilterStackManager;
import org.cresques.io.GeoRasterFile;

/**
 * Gestor de la pila de filtros para el filtro de mediana.
 * 
 * @author Diego Guerrero Sevilla  <diego.guerrero@uclm.es>
 *
 */

public class MedianStackManager implements IStackManager{
	
	public final static int				median = 9;
	protected RasterFilterStack 		filterStack = null;
	private RasterFilterStackManager	filterStackManager = null;
	private GeoRasterFile[] 			grfList = null;
	
	public MedianStackManager(RasterFilterStackManager filterStackManager){
		this.filterStackManager = filterStackManager;
		this.filterStack = filterStackManager.getFilterStack();
		filterStackManager.addTypeFilter("median", MedianStackManager.median, 8);
	}
	
	/**
	 * Obtiene el valor entero que corresponde al tipo de filtro.
	 * @param rasterFilter
	 * @return
	 */
	public int getType(RasterFilter rasterFilter){
		if(rasterFilter instanceof MedianFilter)
			return MedianStackManager.median;
					
		return -1;
	}
	
	/**
	 * A�ade un filtro de mediana a la pila de filtros.
	 * @param ladoVentana
	 */
	public void addMedianFilter(int ladoVentana){
		//Si ya hay un filtro de mediana en la pila lo sobreescribimos
		
		if(filterStack.isActive(((Integer) filterStackManager.getTypeFilters().get("median")).intValue()))
			filterStack.removeFilter(((Integer) filterStackManager.getTypeFilters().get("median")).intValue());
	
		RasterFilter filtro = null;
		
		switch(filterStack.getDataTypeInFilter(((Integer)this.filterStackManager.getTypeFilters().get("median")).intValue())){
		case RasterBuf.TYPE_IMAGE:
			filtro = new MedianImageFilter();
			break;
		case RasterBuf.TYPE_SHORT:
			filtro = new MedianShortFilter();
			break;
		case RasterBuf.TYPE_USHORT:
			filtro = new MedianShortFilter();
			break;
		case RasterBuf.TYPE_INT:
			filtro = new MedianIntegerFilter();
			break;
		case RasterBuf.TYPE_FLOAT:
			filtro = new MedianFloatFilter();
			break;
		case RasterBuf.TYPE_DOUBLE:
			filtro = new MedianDoubleFilter();
			break;
		}
		
		//Cuando el filtro esta creado, tomamos los valores y lo a�adimos a la pila
		
		if(filtro != null){
			filtro.addParam("ladoVentana", new Integer(ladoVentana));
			filterStack.addFilter(((Integer) filterStackManager.getTypeFilters().get("median")).intValue(), filtro);
			filterStackManager.controlTypes();
		}
	}
	
	public void createStackFromStrings(ArrayList f, Integer pos) {
		// TODO Auto-generated method stub
		
	}

	
	/**
     * Crea una pila de filtros a partir de un Array de Strings. Cada elemento del array debe
     * tener la forma elemento=valor.
     * @param filters
     */
	public int createStackFromStrings(ArrayList filters, String fil, GeoRasterFile[] grfList, int filteri) {
		this.grfList = grfList;

			if((fil.startsWith("filter.median.active"))&&
					(RasterFilterStackManager.getValue(fil).equals("true"))){
				
				int ladoVentana = 3;
				filters.remove(0);
				
				for(int prop = 0; prop < filters.size() ; prop++){	
					String elem = (String) filters.get(prop);
					if(elem.startsWith("filter.median.ladoVentana")){
						ladoVentana = Integer.parseInt(RasterFilterStackManager.getValue(elem));
						filters.remove(prop);
						prop--;
					}
				}
				addMedianFilter(ladoVentana);
			}
		//return filteri;
			return -1;
	}

	public ArrayList getStringsFromStack() {
		// TODO Auto-generated method stub
		return null;
	}
	
	/**
	 * Obtiene un Array de Strings a partir de una pila de filtros. Cada elemento del array
	 * tendr� la forma de elemento=valor.
	 */
	
	public ArrayList getStringsFromStack(ArrayList filterList, RasterFilter rf) {
		
		
		if(rf instanceof MedianFilter)
			filterList.add("filter.median.active=true");
		
		if(rf instanceof MedianFilter){
			MedianFilter median = (MedianFilter) rf;
			filterList.add("filter.median.ladoVentana="+median.ladoVentana);
        }
			
		return filterList;
	}

}
