package org.cresques.filter.enhancement;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;

/**
 * Filtro de contraste para imágenes de 16 bits.
 * @author Miguel Ángel Querol Carratalá <querol_mig@gva.es>
 *
 */
public class ContrastShortFilter extends ContrastFilter {
	private short[] 		px = new short[4];
	/**
	 * Variable que contendra el incremento del contraste.
	 */
	private short incrContraste = 0;
	
	private short diferencia = 0;
	
	public ContrastShortFilter(){
		super();
	}
		
	public void pre(){
		exec = true;
		this.raster = (RasterBuf) params.get("raster");
		height = raster.getHeight();
        width = raster.getWidth();
        this.incrContraste = ((Integer)this.params.get("incrContraste")).shortValue();

		super.pre();
	}

	/**
	 * Algoritmo de contraste
	 * @param px3 valores ARGB
	 * @return valores ARGB con el algoritmo aplicado
	 */
	private short[] calcContrast(short[] px){
		short result = 0;
		for (int i = 0 ; i < 3 ; i++){			
			if(incrContraste >= 0){
				if (px[i] < 127){
					diferencia = (short)(127 - px[i]);
					result = (short)(px[i] - (short)((diferencia * 5) * (incrContraste * 0.1) / 25));
					if(result < 0)	result = 0;
					
				}
				if (px[i] > 127){
					diferencia = (short)(px[i] - 127);
					result = (short)(px[i] + (short)((diferencia * 5) * (incrContraste * 0.1) / 25));
					if(result > 255)	result = 255;
				}
			}else{ //incrContraste < 0	
				if(px[i] < 127){
					diferencia = (short)(127 - px[i]);
					result = (short)(px[i] + (diferencia * (short)(-incrContraste * 0.1) / 25));
					if(result > 127)	result = 127;
					if(px[i] == 0 || px[i] == 0)
						if(incrContraste > -255)
							result = 0;
				}
				if(px[i] > 127){
					diferencia = (short)(px[i] - 127);
					result = (short)(px[i] - (diferencia * (short)(-incrContraste * 0.1) / 25));
					if (result < 127)	result = 127;
					if(px[i] == 255 || px[i] == 254)	
						if(incrContraste > -255)
							result = 255;
				}
			}
			px[i] = result;
		}
		return px;
	}
	
	public void process(int col, int line) {
		raster.getElemShort(line, col, px);
		px = calcContrast(px);
		raster.setElemShort(line, col, px);
	}
	
	public void processSuperSampling(int col, int line)  {
		raster.getElemShort(line, col, px);
		px = calcContrast(px);
        for(int j = col; j < raster.getWidth() && j < (col + stepX[contX + 1]); j++)
			for(int i = line; i < raster.getHeight() && i < (line + stepY[contY + 1]); i++)
				 raster.setElemShort(i, j, px);
	}

	public void processLine(int y) {
		// TODO Auto-generated method stub
		
	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_SHORT;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_SHORT;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.raster;
        } else {
            return null;
        }
	}
	
}
