/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*/

package org.cresques.filter.segmentation;

import java.awt.Image;

import org.cresques.filter.RasterFilter;
import org.cresques.io.data.RasterBuf;
import org.gvsig.i18n.Messages;

/**
 * Clase base para los filtros Sobel.
 *
 * @author Diego Guerrero Sevilla  <diego.guerrero@uclm.es>
 *
 */

public abstract class FirstDerivativeFilter extends RasterFilter {
	public static final int TYPE_SOBEL  = 0;
	public static final int TYPE_ROBERTS  = 1;
	public static final int TYPE_PREWITT  = 2;
	public static final int TYPE_FREICHEN  = 3;

	protected int 			umbral = 0;
	protected boolean 		compare = false;
	protected Image  		imageResult = null;
	protected RasterBuf 	rasterResult = null;
	protected Kernel 		operatorH;
	protected Kernel 		operatorV;
	protected int			operator;
	
	// Kernels:------------------------------------------
	double sobelH[][]= {{-1,0,1},{-2,0,2},{-1,0,1}};
	double sobelV[][]= {{-1,-2,-1},{0,0,0},{1,2,1}};
	Kernel sobelHKernel = new Kernel(sobelH);
	Kernel sobelVKernel = new Kernel(sobelV);
	
	double robertsH[][]= {{0,0,-1},{0,1,0},{0,0,0}};
	double robertsV[][]= {{-1,0,0},{0,1,0},{0,0,0}};
	Kernel robertsHKernel = new Kernel(robertsH);
	Kernel robertsVKernel = new Kernel(robertsV);
	
	double prewittH[][]= {{1,0,-1},{1,0,-1},{1,0,-1}};
	double prewittV[][]= {{-1,-1,-1},{0,0,0},{1,1,1}};
	Kernel prewittHKernel = new Kernel(prewittH);
	Kernel prewittVKernel = new Kernel(prewittV);
	
	double freiChenH[][]= {{1,0,-1},{1.4142D,0,-1.4142D},{1,0,-1}};
	double freiChenV[][]= {{-1,-1.4142D,-1},{0,0,0},{1,1.4142D,1}};
	Kernel freiChenHKernel = new Kernel(freiChenH);
	Kernel freiChenVKernel = new Kernel(freiChenV);
	// --------------------------------------------------
	
	
	public FirstDerivativeFilter() {
		super();
	}
	
	/**
	 * Constructor para la asignacin del identificador del filtro
	 * @param fName Cadena que representa el identificador del filtro 
	 */
	public FirstDerivativeFilter(String fName) {
		super();
		super.filterName = fName;
	}

	public void pre() {
		if (params.get("umbral")!=null)
			umbral = ((Integer)params.get("umbral")).intValue();
		else this.umbral = 0;
		
		if (params.get("compare")!=null)
			compare = ((Boolean)params.get("compare")).booleanValue();
		else this.compare = false;
		
		if (params.get("operator")!=null)
			operator = ((Integer)params.get("operator")).intValue();
		else this.operator = 0;
		
		switch (operator){
		case TYPE_SOBEL:
			operatorH=sobelHKernel;
			operatorV=sobelVKernel;
			break;
		case TYPE_ROBERTS:
			operatorH=robertsHKernel;
			operatorV=robertsVKernel;
			break;
		case TYPE_PREWITT:
			operatorH=prewittHKernel;
			operatorV=prewittVKernel;
			break;
		case TYPE_FREICHEN:
			operatorH=freiChenHKernel;
			operatorV=freiChenVKernel;
			break;
		}
	}
	
	public void post() {
		// TODO Auto-generated method stub

	}

	/**
	 * Obtiene el umbral 
	 * @return entero que representa el umbral
	 */
	public int getUmbral() {
		return umbral;
	}

}
