/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*/

package org.cresques.filter.segmentation;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.io.data.RasterBuf;

/**
 * Filtro de Sobel que se aplica en la imagen. Toma como entrada la imagen
 * y el umbral (cero para no umbralizar).
 * 
 * @author Diego Guerrero Sevilla  <diego.guerrero@uclm.es>
 *
 */

public class FirstDerivativeImageFilter extends FirstDerivativeFilter {

	/**
	 * Constructor sin parmetros. Este es usado principalmente desde controlTypes
	 * ya que se instancia con newInstance y sin parmetros. Poseriormente le aplica
	 * el nombre al filtro con la funcin setFilterName de RasterFilter.
	 */
	public FirstDerivativeImageFilter() {}
	
	/**
	 * Constructor para la asignacin del identificador del filtro
	 * @param fName Cadena que representa el identificador del filtro 
	 */
	public FirstDerivativeImageFilter(String fName) {
		super(fName);
	}
	
	public void process(int col, int line) {
		int px;
		int outR,outG,outB;
		int convoResult[][] = new int[3][4];
		int ladoVentana = 3;
		int semiLado = (ladoVentana-1)>>1;
		
		double ventanaR[][]=new double[ladoVentana][ladoVentana];
		double ventanaG[][]=new double[ladoVentana][ladoVentana];
		double ventanaB[][]=new double[ladoVentana][ladoVentana];
		
		Kernel kernelRGB[]=new Kernel[3];

		px = ((BufferedImage) image).getRGB(col, line); 
		int alpha = (px & 0xff000000);       //Extraigo el alpha para mantenerlo.
		
		if((col-semiLado >= 0) && (line-semiLado >= 0) &&(col+semiLado < width)&&(line+semiLado < height)) 
		{
				// Obtener el vector con la ventanas de muestras (una por componente RGB)
			for (int i=-semiLado;i<=semiLado;i++)
				for(int j=-semiLado;j<=semiLado;j++)
				{
					px = ((BufferedImage) image).getRGB(col + i, line + j);
					ventanaR[i+semiLado][j+semiLado] =(px & 0x00ff0000) >> 16;
					ventanaG[i+semiLado][j+semiLado] =(px & 0x0000ff00) >> 8;
					ventanaB[i+semiLado][j+semiLado] =(px & 0x000000ff);
				}
				kernelRGB[0]=new Kernel(ventanaR);
				kernelRGB[1]=new Kernel(ventanaG);
				kernelRGB[2]=new Kernel(ventanaB);
				
				convoResult[0][0]=(int)operatorH.convolution(kernelRGB[0]);
				convoResult[0][1]=(int)operatorV.convolution(kernelRGB[0]);
				convoResult[1][0]=(int)operatorH.convolution(kernelRGB[1]);
				convoResult[1][1]=(int)operatorV.convolution(kernelRGB[1]);
				convoResult[2][0]=(int)operatorH.convolution(kernelRGB[2]);
				convoResult[2][1]=(int)operatorV.convolution(kernelRGB[2]);

				if (compare){
					if (convoResult[0][0]>convoResult[0][1]) outR= convoResult[0][0];
					else outR= convoResult[0][1];
					
					if (convoResult[1][0]>convoResult[1][1]) outG= convoResult[1][0];
					else outG= convoResult[1][1];
					
					if (convoResult[2][0]>convoResult[2][1]) outB= convoResult[2][0];
					else outB= convoResult[2][1];
				}
				else{
					outR=(short)Math.sqrt(Math.pow(convoResult[0][0],2)+Math.pow(convoResult[0][1],2));
					outG=(short)Math.sqrt(Math.pow(convoResult[1][0],2)+Math.pow(convoResult[1][1],2));
					outB=(short)Math.sqrt(Math.pow(convoResult[2][0],2)+Math.pow(convoResult[2][1],2));
				}
				
				if (umbral>0){
					if(outR>=umbral)outR=255;
					else outR=0;
					if(outG>=umbral)outG=255;
					else outG=0;
					if(outB>=umbral)outB=255;
					else outB=0;
				}
				
				else{
					if (outR<0) outR=0;
					else if (outR>255) outR=255;
					if (outG<0) outG=0;
					else if (outG>255) outG=255;
					if (outB<0) outB=0;
					else if (outB>255) outB=255;
				}
			
			((BufferedImage) imageResult).setRGB(col, line, (alpha | 
															((outR << 16) & 0x00ff0000) | 
															((outG << 8) & 0x0000ff00) | 
															(outB & 0x000000ff)));
			
		}
		else ((BufferedImage) imageResult).setRGB(col, line, px);
		

	}

	public void processSuperSampling(int col, int line) {
		int px;
		int outR,outG,outB;
		int convoResult[][] = new int[3][4];
		int ladoVentana = 3;

		Kernel kernelRGB[] = null;

		px = ((BufferedImage) image).getRGB(col, line); 
		int alpha = (px & 0xff000000);       //Extraigo el alpha para mantenerlo.
		
		//		 Obtener el vector con la ventanas de muestras (una por componente RGB)
		kernelRGB=extractSubImage(ladoVentana, col, line);
		if(kernelRGB!=null)
		{
			convoResult[0][0]=(int)operatorH.convolution(kernelRGB[0]);
			convoResult[0][1]=(int)operatorV.convolution(kernelRGB[0]);
			convoResult[1][0]=(int)operatorH.convolution(kernelRGB[1]);
			convoResult[1][1]=(int)operatorV.convolution(kernelRGB[1]);
			convoResult[2][0]=(int)operatorH.convolution(kernelRGB[2]);
			convoResult[2][1]=(int)operatorV.convolution(kernelRGB[2]);
			
			if (compare){
				if (convoResult[0][0]>convoResult[0][1]) outR= convoResult[0][0];
				else outR= convoResult[0][1];
				
				if (convoResult[1][0]>convoResult[1][1]) outG= convoResult[1][0];
				else outG= convoResult[1][1];
				
				if (convoResult[2][0]>convoResult[2][1]) outB= convoResult[2][0];
				else outB= convoResult[2][1];
			}
			else{
				outR=(short)Math.sqrt(Math.pow(convoResult[0][0],2)+Math.pow(convoResult[0][1],2));
				outG=(short)Math.sqrt(Math.pow(convoResult[1][0],2)+Math.pow(convoResult[1][1],2));
				outB=(short)Math.sqrt(Math.pow(convoResult[2][0],2)+Math.pow(convoResult[2][1],2));
			}
				
			if (umbral>0){
				if(outR>=umbral)outR=255;
				else outR=0;
				if(outG>=umbral)outG=255;
				else outG=0;
				if(outB>=umbral)outB=255;
				else outB=0;
			}
			
			else{
				if (outR<0) outR=0;
				else if (outR>255) outR=255;
				if (outG<0) outG=0;
				else if (outG>255) outG=255;
				if (outB<0) outB=0;
				else if (outB>255) outB=255;
			}
			
			 for(int j = col; j < width && j < (col + stepX[contX + 1]); j++)
					for(int i = line; i < height && i < (line + stepY[contY + 1]); i++)	
				((BufferedImage) imageResult).setRGB(j, i, 	alpha | 
												((outR << 16) & 0x00ff0000) |
												((outG << 8) & 0x0000ff00) |
												(outB & 0x000000ff));
	
		}
		else
			 for(int j = col; j < width && j < (col + stepX[contX + 1]); j++)
					for(int i = line; i < height && i < (line + stepY[contY + 1]); i++)	
				((BufferedImage) imageResult).setRGB(j, i,px);

	}

	public void processLine(int y) {
		// TODO Auto-generated method stub

	}

	public int getInRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public int getOutRasterDataType() {
		return RasterBuf.TYPE_IMAGE;
	}

	public Object getResult(String name) {
		if (name.equals("raster")) {
            return (Object) this.imageResult;
        } else {
            return null;
        }
	}
	
	/**
	 * Obtiene tres ventanas centradas en la coordena (x,y) de tama�o ladoVentana * ladoVentana.
	 * 
	 * @return null si la ventana se sale de la imagen.
	 */
	public Kernel[] extractSubImage(int ladoVentana,int x,int y){
		
		int indiceX,indiceY,origenX,origenY;
		int px;
		int semiLado = (ladoVentana-1)>>1;
		int offsetX=0;
		int offsetY=0;
		
		Kernel kernelRGB[] = null;

		if ((contX+1-semiLado>=0) && (contY+1-semiLado>=0)&& (contX+1+semiLado<stepX.length) && (contY+1+semiLado<stepY.length)){
			
			// Calcular el alcance del kernel cuyo centro es (x,y)
			for (int i=0;i<semiLado;i++){
				offsetX=offsetX+stepX[contX+i+1];
				offsetY=offsetY+stepY[contY+i+1];
			}
			if((contY+1-semiLado>=0) && (x+offsetX<width)&&(y+offsetY<height)){
				
				double ventanaR[][] = new double[ladoVentana][ladoVentana];
				double ventanaG[][] = new double[ladoVentana][ladoVentana];
				double ventanaB[][] = new double[ladoVentana][ladoVentana];
				kernelRGB = new Kernel[3];
				
				origenX=x;
				origenY=y;
				
				kernelRGB = new Kernel[3];
				for (int i=0;i<semiLado;i++){
					origenX=origenX-stepX[contX-i];
					origenY=origenY-stepY[contY-i];
				}
				//Recorro el kernel seg�n los step
				indiceX=origenX;
				for (int i=-semiLado;i<=semiLado;i++){
					indiceY=origenY;
					for(int j=-semiLado;j<=semiLado;j++){
						px = ((BufferedImage) image).getRGB(indiceX, indiceY);
						ventanaR[i+semiLado][j+semiLado] =(px & 0x00ff0000) >> 16;
						ventanaG[i+semiLado][j+semiLado] =(px & 0x0000ff00) >> 8;
						ventanaB[i+semiLado][j+semiLado] =(px & 0x000000ff);
						indiceY=indiceY+stepY[contY+j+1];
									
					}
					indiceX=indiceX+stepX[contX+i+1];
				}
				//**************************************************************************************************
				kernelRGB[0]= new Kernel(ventanaR);
				kernelRGB[1]= new Kernel(ventanaG);
				kernelRGB[2]= new Kernel(ventanaB);
			}
		}
		return kernelRGB;
	}

	public void pre() {
		exec = true;
		this.image = (Image) params.get("raster");
		height = image.getHeight(null);
        width = image.getWidth(null);
		imageResult = new BufferedImage(image.getWidth(null),image.getHeight(null), BufferedImage.TYPE_INT_ARGB);
		
		super.pre();
	}

}
