package org.cresques.io.datastruct;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;

/**
 * Clase con herramientas la persistencia de los histogramas calculados.
 * Se implementan mtodos para ecribir y leer en los ficheros .rmf correspondientes
 * los histogramas calculados.
 * @author Miguel ngel Querol Carratal <querol_mig@gva.es>
 *
 */
public class HistogramPersistence {

	
	public HistogramPersistence(){
	}
	
	/**
	 * Crea o modifica el archivo .rmf asociado al GeoRasterFile del que le pasamos
	 * el nombre con los valores del histograma.
	 * @param histogram
	 * @param grfName
	 * @throws IOException
	 */
	public void makeRMFHistogram(int histogram[][], String grfName, int numBands)throws IOException{
		File rmfFile = null;
		File rmfAux = null;
		Writer writer = null;
		boolean copy = true;

		rmfFile = new File(grfName.substring(0, grfName.lastIndexOf(".") + 1) + "rmf");
		if(rmfFile.exists()){
			BufferedReader inGrf = new BufferedReader(new InputStreamReader(new FileInputStream(rmfFile)));
			String str = inGrf.readLine();
			rmfAux = new File(grfName.substring(0, grfName.lastIndexOf(".") + 1) + "aux.rmf");
			rmfAux.createNewFile();
			writer = new FileWriter(rmfAux);
			
			while(str != null){
				if(!str.equals("</RasterMetaFile>")){
					if(str.equals("<Histogram>"))
						copy = false;
					if(str.equals("</Histogram>"))
						copy = true;
					if(copy && (!str.equals("</Histogram>")))
						writer.write(str + "\n");
				}else{
					writer.write(getRMFHistogramHeader());
					for (int i = 0; i < numBands ; i++){
						writer.write(getXMLRasterBandHistogram(histogram, i));
					}
					writer.write(getRMFHistogramEnd());
					writer.write(str + "\n");
				}
				str = inGrf.readLine();
			}
			writer.close();
			
			rmfAux.renameTo(rmfFile);
			
			
		}else{
			rmfFile.createNewFile();
			writer = new FileWriter(rmfFile);
			writer.write(getRMFHeader());
			writer.write(getRMFHistogramHeader());
			for (int i = 0; i < numBands ; i++){
				writer.write(getXMLRasterBandHistogram(histogram, i));
			}
			writer.write(getRMFHistogramEnd());
			writer.write(getRMFEnd());
			writer.close();
		}
	}
	
	
	
	/**
	 * Calcula la cabecera del archivo .rmf asociado a la imagen en el caso
	 * de que ste no exista.
	 * @return
	 */
	private String getRMFHeader(){
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("<?xml version=\"1.0\" encoding=\"ISO-8859-15\"?>\n");
		buffer.append("<RasterMetaFile xmlns=\"http://www.gvsig.org\">\n");
		
		return buffer.toString();
	}
	
	/**
	 * Crea el final del archivo .rmf asociado a la imagen en el caso de que
	 * ste no exista.
	 * @return
	 */
	private String getRMFEnd(){
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("</RasterMetaFile>\n");
		
		return buffer.toString();
	}
	
	/**
	 * Crea la cabecera en XML para el bloque del histograma completo de la
	 * imagen.
	 * @return
	 */
	private String getRMFHistogramHeader(){
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("<Histogram>\n");
		buffer.append("\t<FullHistogram>\n");
		
		return buffer.toString();
	}
	
	/**
	 * Crea el final en XML para el bloque del histograma completo de la
	 * imagen.
	 * @return
	 */
	private String getRMFHistogramEnd(){
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("\t</FullHistogram>\n");
		buffer.append("</Histogram>\n");
		
		return buffer.toString();
	}
	
	/**
	 * Crea el cdigo XML correspondiendte a los datos del histograma de la 
	 * imagen de la banda correspondiente.
	 * @param n_band: banda de la que crear el XML con los datos de su histograma.
	 * @return
	 */
	private String getXMLRasterBandHistogram(int histogram[][], int n_band){
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("\t\t<Band>\n");
		buffer.append("\t\t\t<Values>");
		for (int i = 0 ; i <= 255 ; i++)
			buffer.append(histogram[n_band][i] + " ");
		buffer.append("</Values>\n");
		buffer.append("\t\t</Band>\n");
		
		return buffer.toString();
	}
	
}
