/*
 * Created on 22-jul-2006
 *
 */
/* gvSIG. Sistema de Informaci�n Geogr�fica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.ui.raster;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.cresques.io.datastruct.Histogram;
import org.gvsig.i18n.Messages;

/**
 * Ventana de incremento para la construccin del histograma
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */

public class HistogramIncrement extends Thread {

	private JPanel panel = null;
	private JLabel label = null;
	private HistogramPanel hp = null;
	private JFrame window = new JFrame();
	
	public HistogramIncrement(HistogramPanel hp){
		this.hp = hp;
	}
	
	/**
	 * Muestra la ventana de incremento con el porcentaje de la construccin del
	 * histograma.
	 */
	public void showWindow(){
		if(label != null)
			label.setText(Messages.getText("calculando... ") + 0  + "%");
		window.getContentPane().add(getJPanel());
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		window.setLocation(	((int)d.getWidth()) >> 1, 
	    					((int)d.getHeight()) >> 1);
		window.setSize(200, 60);
		window.show();
		if(this.isAlive())
			this.resume();
		else
			this.start();
	}
	
	/**
	 * Obtiene el panel del interior de la ventana de incremento
	 * @return JPanel
	 */
	private JPanel getJPanel(){
		if(panel == null){
			FlowLayout f = new FlowLayout();
			panel = new JPanel();
			panel.setLayout(f);
			label = new JLabel();
			label.setText(Messages.getText("calculando... ") + 0  + "%");
			panel.add(label, null);
		}
		return panel;
	}
	
	/**
	 * Este thread va leyendo el porcentaje hasta que se completa el histograma.
	 */
	public synchronized void run(){
		while(true){
			Histogram hist = hp.getHistogram();
			while(hist.getPercent() < 100){
				label.setText(Messages.getText("calculando ...") +  hist.getPercent() +"%");
				try {
					sleep(100);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			//Cerramos la ventana
			window.hide();
			window = null;
			
			//Mostramos el resultado
			hp.readFullHistogramFromThread();
			
			this.suspend();

		}
	}
} 
