/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.image.DataBuffer;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.cresques.io.GeoRasterFile;
import org.cresques.io.datastruct.Metadata;
import org.cresques.ui.filter.FilterRasterDialogPanel;

/**
 * 
 * @author Miguel Angel Querol Carratal <querol_mig@gva.es>
 * Panel de informacion de la imagen rster de la que se muestran las propiedades
 *
 */
public class InfoPanel extends JPanel implements IResize{

	
	private int						wComp = 445, hComp = 239;
	private int						wScroll = wComp - 5, hScroll = hComp - 9;
	private int						wEditor = wComp - 20, hEditor = hComp - 19;
	
	private final String bgColor0 = "\"#FEEDD6\""; // light salmon
    private final String bgColor1 = "\"#EAEAEA\""; // light grey
    private final String bgColor2 = "\"#F2FEFF\""; // light blue
    private final String bgColor3 = "\"#FBFFE1\""; // light yellow
	private JScrollPane jScrollPane = null;
	public JEditorPane jEditorPane = null;
    
	/**
	 * Nombre del Panel
	 */
	public String nom = "Info";
	
	/**
	 * Dialogo padre que contiene a este
	 */
	public FilterRasterDialogPanel parent = null;
	
	/**
	 * Matriz de tamao Nx2 que contiene a las propiedades en forma propiedad/valor
	 */
	public Object[][] 					props = null;
	
	/**
	 * Cabecera de las columnas del dialogo
	 */
	public Object[] columnNames = { "Propiedad", "Valor" };
	
	/**
	 * Cadenas de caracterres que corresponden con las cabeceras de las tablas
	 * de informacin
	 */
	public String 						cabInfo = null;
	public String 						cabCoord = null;
	public String 						cabProy = null;
	public String 						cabOrig = null;
	public String						cabMeta = null;
	/**
	 * Vector de GeoRasterFiles para almacenar los archivos que forman
	 * parte de la capa.
	 */
	private GeoRasterFile[]				files = null;
	
	/**
	 * Flags que indican que archivo corresponde a cada banda de color
	 */
	private int							redBand = -1;
	private int							greenBand = -1;
	private int							blueBand = -1;
	
	
	
	/**
	 * This is the default constructor
	 */
	public InfoPanel(FilterRasterDialogPanel parent) {
		super();
		this.files = parent.getGrf();
		this.parent = parent;
		this.props = parent.props;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	public void initialize() {
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints.gridx = 0;
		gridBagConstraints.gridy = 0;
		gridBagConstraints.weightx = 1.0;
		gridBagConstraints.weighty = 1.0;
		gridBagConstraints.insets = new java.awt.Insets(5,8,5,8);
		this.setLayout(new GridBagLayout());
		//this.setSize(wComp, hComp);
		//this.setPreferredSize(new java.awt.Dimension(wComp,hComp));
		this.add(getJScrollPane(), gridBagConstraints);
		//refresh();
		this.setComponentSize(wComp, hComp);
		this.getJEditorPane().repaint();
		
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			//jScrollPane.setPreferredSize(new java.awt.Dimension(wScroll,hScroll));
			jScrollPane.setViewportView(getJEditorPane());
			

		}
		return jScrollPane;
	}

	/**
	 * This method initializes jEditorPane	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */
	private JEditorPane getJEditorPane() {
		if (jEditorPane == null) {
			jEditorPane = new JEditorPane();
			jEditorPane.setEditable(false);
			//jEditorPane.setSize(new java.awt.Dimension(wEditor,hEditor));
			
		}
		return jEditorPane;
	}
	
	public void resetTable(){
		this.jEditorPane = null;
	}
	
	/**
	 * Obtiene el nombre del panel.
	 */
	public String getName(){
    	return this.nom;
    }
		
	/**
	 * Aade los GeoRasterFiles que forman parte de la capa
	 * @param Vector de GeoRasterFiles que corresponde con los
	 * archivos cargados.
	 */
	public void addFiles(GeoRasterFile[] files){
		this.files = files;
		refresh();
	}
	
	/**
	 * Asigna el nmero de lnea del archivo que corresponde a cada bada de color
	 * @param bandR Nmero de linea del archivo que corresponde a la banda roja
	 * @param bandG Nmero de linea del archivo que corresponde a la banda verde
	 * @param bandB Nmero de linea del archivo que corresponde a la banda azul
	 */
	public void setBands(int bandR, int bandG, int bandB){
		this.redBand = bandR;
		this.greenBand = bandG;
		this.blueBand = bandB;
		refresh();
	}
	
	
	/**
	 * Mtodo que crea el cdigo HTML para la tabla de informacin general
	 * dentro del panel de informacin de raster
	 * @return String con el cfigo HTML que corresponde con la tabla de informacin
	 * general
	 */
	public String tablaInfo(){
		String tablaInfo = null;
		String propiedades="";
		String color = "\"#FEEDD6\"";
		
		String cabecera = 
			"  <tr valign=\"top\">" +
            "     <td width=\"417\" height=\"18\" bgcolor="+bgColor3+"align=\"center\" colspan=\"2\"><font face=\"Arial\" size=\"3\" align=\"right\"><b>"+cabInfo+"</b></font></td>" +
            "  </tr>";
		
		for(int i = 0; i <= 5; i++){
			
			if ((i%2 == 0) || i == 0) color = bgColor0;
			else color = bgColor1;
			
			//if(props[i][1] == null) props[i][1] = "";
			
			if(props.length > 1){
				String propiedad = 
					"  <tr valign=\"top\">" +
					"     <td width=\"140\" height=\"18\" bgcolor=\"#D6D6D6\"align=\"right\"><font face=\"Arial\" size=\"3\" align=\"right\">"+props[i][0]+"</font></td>" +
					"     <td width=\"270\" height=\"18\" bgcolor="+color+"align=\"left\"><font face=\"Arial\" size=\"3\">"+props[i][1]+"</font></td>" +
					"  </tr>";
				propiedades = propiedades + propiedad; 
			}
		}
		
		tablaInfo = "<table>" + cabecera + propiedades + "<tr></tr>" + "</table>";
		
		return tablaInfo;
	}
	
	
	
	
	/**
	 * Mtodo que crea el cdigo HTML para la tabla de coordenadas geogrficas
	 * dentro del panel de informacin de raster
	 * @return String con el cfigo HTML que corresponde con la tabla de coordenadas
	 * geogrficas.
	 */
	public String tablaCoord(){
		String tablaCoord = null;
		String propiedades = "";
		String color = "\"#FEEDD6\"";
		
		if(props.length > 1){
			String cabecera = 
				"  <tr valign=\"top\">" +
	            "     <td width=\"417\" height=\"18\" bgcolor="+bgColor3+"align=\"center\" colspan=\"2\"><font face=\"Arial\" size=\"3\" align=\"right\"><b>"+cabCoord+"</b></font></td>" +
	            "  </tr>";
			
			for(int i = 7; i <= 12; i++){
				
				if ((i%2 == 1) || i == 7) color = bgColor0;
				else color = bgColor1;
				
				if(props[i][1] == null) props[i][1] = "";
				
				if(props[i][0] != null){
					String propiedad = 
						"  <tr valign=\"top\">" +
						"     <td width=\"140\" height=\"18\" bgcolor=\"#D6D6D6\"align=\"right\"><font face=\"Arial\" size=\"3\" align=\"right\">"+props[i][0]+"</font></td>" +
						"     <td width=\"270\" height=\"18\" bgcolor="+color+"align=\"left\"><font face=\"Arial\" size=\"3\">"+props[i][1]+"</font></td>" +
						"  </tr>";
					propiedades = propiedades + propiedad; 
				}
			}
			tablaCoord = "<table>" + cabecera + propiedades + "<tr></tr>" + "</table>";
			
			return tablaCoord;
			}
		return "";
	}

	
	
	/**
	 * Mtodo que crea la tabla de origen de datos dentro del panel
	 * de Informacin.
	 * @param vector con todos los GeoRasterFiles que estn dentro de la capa
	 * @return String con el cdigo HTML de la tabla.
	 */
	public String tablaOrigen(GeoRasterFile[] files){
		String tablaOrigen = null;
		String propiedades = "";
		String color = "\"#FEEDD6\"";
		String rBand = "";
		String gBand = "";
		String bBand = "";
		String bandType = "";
		int k = 0;
		
		
		String cabecera = 
			"  <tr valign=\"top\">" +
			"     <td width=\"417\" height=\"18\" bgcolor="+bgColor3+"align=\"center\" colspan=\"4\"><font face=\"Arial\" size=\"3\" align=\"right\"><b>"+cabOrig+"</b></font></td>" +
			"  </tr>";
		
		for(int i = 0; i<files.length ; i++){
			String fName = files[i].getName();
			String propiedad = 
				"  <tr valign=\"top\">" +
				"     <td width=\"417\" height=\"18\" bgcolor="+bgColor1+"align=\"left\" colspan=\"4\"><font face=\"Arial\" size=\"3\" align=\"right\">"+fName+"</font></td>" +
				"  </tr>";
			
			switch (files[i].getDataType()) {
            case DataBuffer.TYPE_BYTE:
                bandType = "8U";
                break;
            case DataBuffer.TYPE_INT:
                bandType = "32";
                break;
            case DataBuffer.TYPE_DOUBLE:
                bandType = "64";
                break;
            case DataBuffer.TYPE_FLOAT:
                bandType = "32";
                break;
            case DataBuffer.TYPE_SHORT:
                bandType = "16";
                break;
            case DataBuffer.TYPE_USHORT:
                bandType = "16U";
                break;
            case DataBuffer.TYPE_UNDEFINED:
                bandType = "??";
                break;
            }
			
			for(int j = 0; j < files[i].getBandCount() ; j++){
				if ((j%2 == 0) || j == 0) color = bgColor0;
				else color = bgColor1;
				
				if(k == redBand) rBand = "R"; 		else rBand = "";
				if(k == greenBand) gBand = "G";		else gBand = "";
				if(k == blueBand) bBand = "B";		else bBand = "";
				
				String banda = 
					"  <tr valign=\"top\">" +
					"     <td width=\"20\" height=\"18\" bgcolor=\"#D6D6D6\"align=\"center\" colspan=\"1/2\"><font face=\"Arial\" size=\"3\" align=\"right\">"+rBand+"</font></td>" +
					"     <td width=\"20\" height=\"18\" bgcolor=\"#D6D6D6\"align=\"center\" colspan=\"1/2\"><font face=\"Arial\" size=\"3\" align=\"right\">"+gBand+"</font></td>" +
					"     <td width=\"20\" height=\"18\" bgcolor=\"#D6D6D6\"align=\"center\" colspan=\"1/2\"><font face=\"Arial\" size=\"3\" align=\"right\">"+bBand+"</font></td>" +
					"     <td width=\"357\" height=\"18\" bgcolor="+color+"align=\"left\"><font face=\"Arial\" size=\"3\">"+"band "+(j+1)+"["+bandType+"]"+"</font></td>" +
					"  </tr>";
				propiedad = propiedad + banda;
				k++;
			}
			propiedades = propiedades + propiedad;
		}
		
		tablaOrigen = "<table>" + cabecera + propiedades + "<tr></tr>" + "</table>";
		
		return tablaOrigen;
	}
	
	
	/**
	 * Mtodo para crear la tabla de informacin de metadatos del raster en el
	 * panel de informacin de propiedades de raster.
	 * @param Vector con los georasterfiles cargados en la capa.
	 * @return String con el cdigo HTML que genera la tabla.
	 */
	public String tablaMetadatos(GeoRasterFile[] files){
		String tablaMetadatos = "";
		String propiedades = "";
		String[] metadatos = null;
		String color = "\"#FEEDD6\"";
		
		Metadata meta = null;
		boolean datos = false;
		
		String cabecera = 
			"  <tr valign=\"top\">" +
			"     <td width=\"417\" height=\"18\" bgcolor="+bgColor3+"align=\"center\" colspan=\"4\"><font face=\"Arial\" size=\"3\" align=\"right\"><b>"+cabMeta+"</b></font></td>" +
			"  </tr>";
		
		for (int i = 0 ; i < files.length ; i++){
			meta = files[i].getMetadata();
			if (meta != null){
				metadatos = meta.getMetadataString();
				String fichero = 
					"  <tr valign=\"top\">" +
					"     <td width=\"417\" height=\"18\" bgcolor="+bgColor1+"align=\"left\" colspan=\"4\"><font face=\"Arial\" size=\"3\" align=\"right\">"+files[i].getName()+"</font></td>" +
					"  </tr>";
				propiedades = propiedades + fichero;
				for(int j = 0 ; j<metadatos.length ; j++){
					datos = true;
					if ((j%2 == 0) || j == 0) color = bgColor0;
					else color = bgColor1;
					int index = metadatos[j].indexOf("=");
					
					String propiedad = 
						"  <tr valign=\"top\">" +
						"     <td width=\"140\" height=\"18\" bgcolor=\"#D6D6D6\"align=\"right\"><font face=\"Arial\" size=\"3\" align=\"right\">"+metadatos[j].substring(0,index)+"</font></td>" +
						"     <td width=\"270\" height=\"18\" bgcolor="+color+"align=\"left\"><font face=\"Arial\" size=\"3\">"+metadatos[j].substring(index+1)+"</font></td>" +
						"  </tr>";
					propiedades = propiedades + propiedad; 
				}
			}
		}
		
		tablaMetadatos = cabecera + propiedades;
		if(datos == false)
			return "";
		
		return tablaMetadatos;
	}
	
	/**
	 * Mtodo que dibuja las tablas HTML del panel de informacin dentro de las
	 * propiedades de rster. Se llama cada vez que se actualiza algn dato de las 
	 * tablas.
	 *
	 */
	public void refresh(){
		String html = null;
		
		html = 
			"<html>"+
			this.tablaInfo()+
			this.tablaCoord();
		
		if(this.files != null){
			html = html + this.tablaOrigen(files) + this.tablaMetadatos(files) +"</html>";
		}
		
		else
			html = html + "</html>";
			
		this.getJEditorPane().setContentType("text/html");
		this.getJEditorPane().setText(html);
		this.getJScrollPane().setAlignmentY(0);
	}

	
	public void setComponentSize(int w, int h){
		wComp = w; hComp = h;
		wScroll = wComp - 5; hScroll = hComp - 9;
		wEditor = wComp - 20; hEditor = hComp - 19;
		
		this.setSize(wComp, hComp);
		this.setPreferredSize(new java.awt.Dimension(wComp,hComp));
		jScrollPane.setPreferredSize(new java.awt.Dimension(wScroll,hScroll));
		jEditorPane.setSize(new java.awt.Dimension(wEditor,hEditor));
			
	}
	
}


