package org.cresques.ui.raster.listener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Rectangle2D;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import org.cresques.io.data.ICutRaster;
import org.cresques.ui.raster.CutRasterPanel;
import org.cresques.util.Utilities;

/**
 * Listener para el panel de recorte de imagen.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class CutRasterListener extends JPanel implements ActionListener, KeyListener, FocusListener{

	private CutRasterPanel	panel = null;
	private Rectangle2D 	extent = null;
	private int 			widthPx = 0;
	private int 			heightPx = 0;
	/**
	 * Clase que instancia a esta y que implemente ITailRaster.
	 */
	private ICutRaster					parent = null;


	/**
	 * Contructor
	 * @param panel
	 */
	public CutRasterListener(CutRasterPanel panel, Rectangle2D extent, int widthPx, int heightPx, ICutRaster parent){
		this.panel = panel;
		this.extent = extent;
		this.widthPx = widthPx;
		this.heightPx = heightPx;
		this.parent = parent;

		panel.getPCoordPixel().getPDataInput11().getJTextField().addKeyListener(this);
		panel.getPCoordPixel().getPDataInput11().getJTextField().addFocusListener(this);
		panel.getPCoordPixel().getPDataInput12().getJTextField().addKeyListener(this);
		panel.getPCoordPixel().getPDataInput12().getJTextField().addFocusListener(this);
		panel.getPCoordPixel().getPDataInput21().getJTextField().addKeyListener(this);
		panel.getPCoordPixel().getPDataInput21().getJTextField().addFocusListener(this);
		panel.getPCoordPixel().getPDataInput22().getJTextField().addKeyListener(this);
		panel.getPCoordPixel().getPDataInput22().getJTextField().addFocusListener(this);

		panel.getPCoordReal().getPDataInput11().getJTextField().addKeyListener(this);
		panel.getPCoordReal().getPDataInput11().getJTextField().addFocusListener(this);
		panel.getPCoordReal().getPDataInput12().getJTextField().addKeyListener(this);
		panel.getPCoordReal().getPDataInput12().getJTextField().addFocusListener(this);
		panel.getPCoordReal().getPDataInput21().getJTextField().addKeyListener(this);
		panel.getPCoordReal().getPDataInput21().getJTextField().addFocusListener(this);
		panel.getPCoordReal().getPDataInput22().getJTextField().addKeyListener(this);
		panel.getPCoordReal().getPDataInput22().getJTextField().addFocusListener(this);

		panel.getPResolution().getCCellSize().getJTextField().addKeyListener(this);
		panel.getPResolution().getCCellSize().getJTextField().addFocusListener(this);
		panel.getPResolution().getCWidth().getJTextField().addKeyListener(this);
		panel.getPResolution().getCWidth().getJTextField().addFocusListener(this);
		panel.getPResolution().getCHeight().getJTextField().addKeyListener(this);
		panel.getPResolution().getCHeight().getJTextField().addFocusListener(this);

		panel.getPResize().getButton(0).addActionListener(this);
		panel.getPResize().getButton(1).addActionListener(this);
		panel.getPResize().getButton(2).addActionListener(this);
	}

	/**
	 * Comprueba si todos los campos de coordenadas estn llenos
	 * @return true si todos los campos de coordenadas estn llenos y false si hay alguno vacio.
	 */
	private boolean isFieldsFill(){
		if(	panel.getPCoordPixel().getPDataInput11().getJTextField().equals("") ||
			panel.getPCoordPixel().getPDataInput12().getJTextField().equals("") ||
			panel.getPCoordPixel().getPDataInput21().getJTextField().equals("") ||
			panel.getPCoordPixel().getPDataInput22().getJTextField().equals("") ||
			panel.getPCoordReal().getPDataInput11().getJTextField().equals("") ||
			panel.getPCoordReal().getPDataInput12().getJTextField().equals("") ||
			panel.getPCoordReal().getPDataInput21().getJTextField().equals("") ||
			panel.getPCoordReal().getPDataInput22().getJTextField().equals(""))
			return false;
		return true;
	}

	/**
	 * Obtiene un rectangulo con la ventana en pixeles. Estos datos son leidos desde el interfaz.
	 * @return Rectangle2D
	 */
	private Rectangle2D getPxWindow(){
		if(panel.getPxMinX() == 0 && panel.getPxMinY() == 0 && panel.getPxMaxX() == 0 && panel.getPxMaxY() == 0)
			return null;
		return new Rectangle2D.Double(panel.getPxMinX(), panel.getPxMinY(), Math.abs(panel.getPxMaxX() - panel.getPxMinX()), Math.abs(panel.getPxMaxY() - panel.getPxMinY()));
	}

	/**
	 * Obtiene un rectangulo con la ventana en coordenadas reales. Estos datos son leidos desde el interfaz.
	 * @return Rectangle2D
	 */
	private Rectangle2D getMapWindow(){
		if(panel.getWcMinX() == 0 && panel.getWcMinY() == 0 && panel.getWcMaxX() == 0 && panel.getWcMaxY() == 0)
			return null;
		return new Rectangle2D.Double(panel.getWcMinX(), panel.getWcMaxY(), Math.abs(panel.getWcMaxX() - panel.getWcMinX()), Math.abs(panel.getWcMaxY() - panel.getWcMinY()));
	}

	/**
	 * Recalcula el valor de los campos de coordenadas reales y pixel. Cuando modificamos algn campo
	 * de las coordenadas reales se modifican los pixeles y viceversa.
	 * @param modifyPx true si se ha modificado algn campo de coordenadas pixel y false si se ha modificado
	 * algn campo de las coordenadas reales.
	 */
	private void recalcCoordFields(boolean modifyPx){
		if(isFieldsFill()){
			try{
				if(modifyPx){
					Rectangle2D pxWindow = getPxWindow();
					if(pxWindow == null)
						return;
					Rectangle2D newWC = Utilities.getMapRectFromPxRect(extent, widthPx, heightPx, pxWindow);
					panel.setCoorRealFromDouble(newWC.getMinX(), newWC.getMaxY(), newWC.getMaxX(), newWC.getMinY(), 6);
					panel.setWidthText(pxWindow.getWidth(), 0);
					panel.setHeightText(pxWindow.getHeight(), 0);
					panel.setCellSizeText((newWC.getWidth() / pxWindow.getWidth()), 4);
					panel.setRelWidthHeight((double)(pxWindow.getWidth() / pxWindow.getHeight()));
					parent.setInitValuesToSize(pxWindow.getWidth(), pxWindow.getHeight());
				}else{
					Rectangle2D mapWindow = getMapWindow();
					if(mapWindow == null)
						return;
					Rectangle2D newPx = Utilities.getPxRectFromMapRect(extent, widthPx, heightPx, mapWindow);
					panel.setCoorPixelFromDouble(newPx.getMinX(), newPx.getMinY(), newPx.getMaxX(), newPx.getMaxY(), 3);
					panel.setWidthText(newPx.getWidth(), 0);
					panel.setHeightText(newPx.getHeight(), 0);
					panel.setCellSizeText((mapWindow.getWidth() / newPx.getWidth()), 4);
					panel.setRelWidthHeight((double)(newPx.getWidth() / newPx.getHeight()));
					parent.setInitValuesToSize(newPx.getWidth(), newPx.getHeight());
				}

			}catch(NumberFormatException ex){
				return;
			}
		}
	}


	public void actionPerformed(ActionEvent e) {
		//Botn de seleccin del extent completo
		//Modificamos las coordenadas reales y recalculamos las coordenadas pixel
		if (e.getSource() == panel.getPResize().getButton(0)){
			panel.selectLayerFullExtend();
		}

		//Botn de seleccin de la herramienta de seleccionar desde la vista
		if (e.getSource() == panel.getPResize().getButton(1)){
			parent.selectToolButton();
		}

		//Botn de contraer y expandir el panel
		if (e.getSource() == panel.getPResize().getButton(2))
			panel.contractExpandPanel();
	}

	public void keyPressed(KeyEvent e) {
		// TODO Auto-generated method stub

	}

	/**
	 * Al producirse un evento de perdida de foco o pulsacin de "enter" en un campo de texto de coordenadas
	 * hay que asignar el nuevo valor introducido.
	 * @param obj
	 */
	private void eventJTextField(Object obj){
		try{
			if(	obj == panel.getPCoordPixel().getPDataInput11().getJTextField())
				panel.setCoorPixelFromDouble(Double.valueOf(panel.getPCoordPixel().getPDataInput11().getJTextField().getText()).doubleValue(),
						panel.getPxMinY(), panel.getPxMaxX(), panel.getPxMaxY(), 3);
			if(	obj == panel.getPCoordPixel().getPDataInput12().getJTextField())
				panel.setCoorPixelFromDouble(panel.getPxMinX(), Double.valueOf(panel.getPCoordPixel().getPDataInput12().getJTextField().getText()).doubleValue(),
						panel.getPxMaxX(), panel.getPxMaxY(), 3);
			if(	obj == panel.getPCoordPixel().getPDataInput21().getJTextField())
				panel.setCoorPixelFromDouble(panel.getPxMinX(), panel.getPxMinY(),
						Double.valueOf(panel.getPCoordPixel().getPDataInput21().getJTextField().getText()).doubleValue(), panel.getPxMaxY(), 3);
			if(	obj == panel.getPCoordPixel().getPDataInput22().getJTextField())
				panel.setCoorPixelFromDouble(panel.getPxMinX(), panel.getPxMinY(),
						panel.getPxMaxX(), Double.valueOf(panel.getPCoordPixel().getPDataInput22().getJTextField().getText()).doubleValue(), 3);

			if(	obj == panel.getPCoordReal().getPDataInput11().getJTextField())
				panel.setCoorRealFromDouble(Double.valueOf(panel.getPCoordReal().getPDataInput11().getJTextField().getText()).doubleValue(),
						panel.getWcMinY(), panel.getWcMaxX(), panel.getWcMaxY(), 6);
			if(	obj == panel.getPCoordReal().getPDataInput12().getJTextField())
				panel.setCoorRealFromDouble(panel.getWcMinX(), Double.valueOf(panel.getPCoordReal().getPDataInput12().getJTextField().getText()).doubleValue(),
						panel.getWcMaxX(), panel.getWcMaxY(), 6);
			if(	obj == panel.getPCoordReal().getPDataInput21().getJTextField())
				panel.setCoorRealFromDouble(panel.getWcMinX(), panel.getWcMinY(),
						Double.valueOf(panel.getPCoordReal().getPDataInput21().getJTextField().getText()).doubleValue(), panel.getWcMaxY(), 6);
			if(	obj == panel.getPCoordReal().getPDataInput22().getJTextField())
				panel.setCoorRealFromDouble(panel.getWcMinX(), panel.getWcMinY(),
						panel.getWcMaxX(), Double.valueOf(panel.getPCoordReal().getPDataInput22().getJTextField().getText()).doubleValue(), 6);
		}catch(NumberFormatException ex1){
			//No hay valores parseables a decimal en las cajas de texto. No hacemos nada
		}
	}

	/**
	 * Cuando se presiona la tecla enter sobre los campos de texto se recalculan las
	 * coordenadas. Si se ha presionado sobre los cuadro correspondientes a las coordenadas
	 * pixel se recalculan las coordenadas del mundo real y viceversa.
	 */
	public void keyReleased(KeyEvent e) {
		if(e.getKeyCode() == 10){
			if(	e.getSource() == panel.getPCoordPixel().getPDataInput11().getJTextField() ||
				e.getSource() == panel.getPCoordPixel().getPDataInput12().getJTextField() ||
				e.getSource() == panel.getPCoordPixel().getPDataInput21().getJTextField() ||
				e.getSource() == panel.getPCoordPixel().getPDataInput22().getJTextField()){
				eventJTextField(e.getSource());
				recalcCoordFields(true);
			}
			if(	e.getSource() == panel.getPCoordReal().getPDataInput11().getJTextField() ||
				e.getSource() == panel.getPCoordReal().getPDataInput12().getJTextField() ||
				e.getSource() == panel.getPCoordReal().getPDataInput21().getJTextField() ||
				e.getSource() == panel.getPCoordReal().getPDataInput22().getJTextField()){
				eventJTextField(e.getSource());
				recalcCoordFields(false);
			}
			if(e.getSource() == panel.getPResolution().getCCellSize().getJTextField()){
				//Cambiamos PS ==> wPx=wWC/PS  &   hPx=wPx/rel
				double ps = 0;
				try{
					ps = Double.parseDouble(panel.getPResolution().getCCellSize().getJTextField().getText());
				}catch(NumberFormatException ex){
					return;
				}
				Rectangle2D mapWindow = getMapWindow();
				Rectangle2D pxWindow = getPxWindow();
				if(mapWindow == null || pxWindow ==  null)
					return;
				panel.setWidthText(mapWindow.getWidth() / ps, 0);
				panel.setHeightText((mapWindow.getWidth() / ps) / panel.getRelWidthHeight(), 0);
			}

			if(e.getSource() == panel.getPResolution().getCWidth().getJTextField()){
				//Cambiamos wPx ==> hPx=wPx/rel  &   PS=wWC/wPx
				double wPx = 0;
				try{
					wPx = Double.parseDouble(panel.getPResolution().getCWidth().getJTextField().getText());
				}catch(NumberFormatException ex){
					return;
				}
				Rectangle2D mapWindow = getMapWindow();
				Rectangle2D pxWindow = getPxWindow();
				if(mapWindow == null || pxWindow ==  null)
					return;
				panel.setWidthText(wPx, 0);
				panel.setHeightText(wPx / panel.getRelWidthHeight(), 0);
				panel.setCellSizeText((mapWindow.getWidth() / wPx), 4);
			}

			if(e.getSource() == panel.getPResolution().getCHeight().getJTextField()){
				//Cambiamos hPx ==> wPx=rel*wPx  &   PS=hWC/hPx
				double hPx = 0;
				try{
					hPx = Double.parseDouble(panel.getPResolution().getCHeight().getJTextField().getText());
				}catch(NumberFormatException ex){
					return;
				}
				Rectangle2D mapWindow = getMapWindow();
				Rectangle2D pxWindow = getPxWindow();
				if(mapWindow == null || pxWindow ==  null)
					return;
				panel.setWidthText(hPx * panel.getRelWidthHeight(), 0);
				panel.setHeightText(hPx, 0);
				panel.setCellSizeText((mapWindow.getHeight() / hPx), 4);
			}
		}
	}

	public void keyTyped(KeyEvent e) {

	}

	public void focusGained(FocusEvent e) {
		// TODO Auto-generated method stub

	}

	/**
	 * Cuando se pierde el foco alguno de los campos de texto se recalculan las
	 * coordenadas. Si lo han perdido los cuadros correspondientes a las coordenadas
	 * pixel se recalculan las coordenadas del mundo real y viceversa.
	 */
	public void focusLost(FocusEvent e) {
		if(	e.getSource() == panel.getPCoordPixel().getPDataInput11().getJTextField() ||
				e.getSource() == panel.getPCoordPixel().getPDataInput12().getJTextField() ||
				e.getSource() == panel.getPCoordPixel().getPDataInput21().getJTextField() ||
				e.getSource() == panel.getPCoordPixel().getPDataInput22().getJTextField()){
				eventJTextField(e.getSource());
				recalcCoordFields(true);
		}
		if(	e.getSource() == panel.getPCoordReal().getPDataInput11().getJTextField() ||
			e.getSource() == panel.getPCoordReal().getPDataInput12().getJTextField() ||
			e.getSource() == panel.getPCoordReal().getPDataInput21().getJTextField() ||
			e.getSource() == panel.getPCoordReal().getPDataInput22().getJTextField()){
			eventJTextField(e.getSource());
			recalcCoordFields(false);
		}
	}


}
