/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*/
package org.cresques.ui.raster.listener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.event.TableModelEvent;

import org.cresques.io.datastruct.Histogram;
import org.cresques.io.datastruct.Statistic;
import org.cresques.ui.raster.HistogramPanel;

/**
 * Listener para eventos del panel de histograma
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 */
public class HistogramPanelListener implements FocusListener, KeyListener, MouseListener, ActionListener, MouseMotionListener{

	private HistogramPanel			panel = null;
	/**
	 * Objeto histograma para la gestin de procesos de histograma
	 */
	private	Histogram				histogramObj = null;
	
	public static boolean 			comboEventEnable = true;
		
	public HistogramPanelListener(HistogramPanel p){
		panel = p;
	}
	
	public void setControlListeners(){
		panel.getPHistogram().getPlusButtonControlLeft().addActionListener(this);
		panel.getPHistogram().getPlusButtonControlRight().addActionListener(this);
		panel.getPHistogram().getLessButtonControlLeft().addActionListener(this);
		panel.getPHistogram().getLessButtonControlRight().addActionListener(this);
		panel.getPHistogram().getTextControlRight().addKeyListener(this);
		panel.getPHistogram().getTextControlLeft().addKeyListener(this);
		panel.getPHistogram().getTextControlRight().addFocusListener(this);
		panel.getPHistogram().getTextControlLeft().addFocusListener(this);
	}
	
	public void mouseDragged(MouseEvent e) {
				
	}

	public void mousePressed(MouseEvent e) {
		
	}

	public void mouseReleased(MouseEvent e) {
			
	}
	
	public void setLabels(JLabel left, JLabel right){
		  
	}
	
	public void mouseMoved(MouseEvent e) {
			
	}

	private void initialize(){

	}
	
	public void actionPerformed(ActionEvent e) {
		
		//--------------------------------------
		//Botones de ms y menos
		if(	e.getSource() == panel.getPHistogram().getPlusButtonControlLeft() ||
			e.getSource() == panel.getPHistogram().getLessButtonControlLeft() ||
			e.getSource() == panel.getPHistogram().getPlusButtonControlRight() ||
			e.getSource() == panel.getPHistogram().getLessButtonControlRight()){
			panel.setStatistic(Statistic.getBasicStatsFromHistogram(panel.getLastHistogram(),
																	(int)panel.getBoxesValues()[1], 
																	(int)panel.getBoxesValues()[0],
																	panel.showBands));
			return;
		}
			
		
		//--------------------------------------
		//Cambiar las bandas en el combo
		JComboBox cbb = panel.getJComboBands();
		if(comboEventEnable && e.getSource() == cbb){
			if(cbb.getSelectedIndex() == 0){
				boolean[] list = panel.getBandListShowInChart();
				for(int i = 0; i < list.length; i++)
					list[i] = true;
				panel.setBandListShowInChart(list);
			}else if(cbb.getSelectedItem().equals("R") || cbb.getSelectedItem().equals("Band 0"))
				panel.addOrRemoveGraphicBand(0);
			else if(cbb.getSelectedItem().equals("G") || cbb.getSelectedItem().equals("Band 1"))
				panel.addOrRemoveGraphicBand(1);
			else if(cbb.getSelectedItem().equals("B") || cbb.getSelectedItem().equals("Band 2"))
				panel.addOrRemoveGraphicBand(2);
			else{
				for(int i = 3; i < HistogramPanel.MAXBANDS; i++){
					if(cbb.getSelectedItem().equals("Band "+i))
						panel.addOrRemoveGraphicBand(i);
				}
			}
		}
		
		//--------------------------------------		
		//Limpiar
		JButton clean = panel.getJButtonClear();
		if(e.getSource() == clean){
			panel.cleanChart();
		}
		
		//--------------------------------------
		//Seleccin de fuente de datos del histograma
		JComboBox cbo = panel.getJComboBoxOrigen();
		if(comboEventEnable && e.getSource() == cbo){
			panel.setHistogramDataSource(cbo.getSelectedIndex());
			
			//En caso de que el histograma se monte a partir de los datos de la vista ponemos RGB en el combo
			if(cbo.getSelectedIndex() == 0)
				panel.setRGBInBandList();
			
			//En caso de que el histograma se monte a partir de los datos reales ponemos el nmero de bandas en el combo
			if(cbo.getSelectedIndex() == 1 || cbo.getSelectedIndex() == 2)
				panel.setBands(panel.getHistogram().getGrid().getBandCount());			
		}
				
		//--------------------------------------
		//Seleccin de histograma acumulado y no acumulado
		JComboBox cbt = panel.getJComboBoxTipo();
		if(comboEventEnable && e.getSource() == cbt)
			panel.setType(cbt.getSelectedIndex());
	
		if(comboEventEnable)
			panel.showHistogram();
	}

	public void tableChanged(TableModelEvent e) {
				
	}

	public void mouseClicked(MouseEvent e) {
		
	}

	public void mouseEntered(MouseEvent e) {
				
	}

	public void mouseExited(MouseEvent e) {
				
	}

	/**
	 * Asigna el objeto histograma para la gestin de procesos de histograma
	 * @param histogramObj
	 */
	public void setHistogram(Histogram histogramObj) {
		this.histogramObj = histogramObj;
	}

	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode() == 10){
			panel.setStatistic(Statistic.getBasicStatsFromHistogram(panel.getLastHistogram(),
																	(int)panel.getBoxesValues()[1], 
																	(int)panel.getBoxesValues()[0],
																	panel.showBands));
			
		}
	}

	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void focusGained(FocusEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void focusLost(FocusEvent e) {
		// TODO Auto-generated method stub
		
	}

}
