package com.iver.cit.gvsig.fmap.drivers.gml;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.types.IXMLType;

import com.hardcode.gdbms.engine.values.ComplexValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.io.gml2.GMLReader;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FMAPGeometryFactory.java,v 1.1.4.2 2007/01/25 16:11:35 jorpiell Exp $
 * $Log: FMAPGeometryFactory.java,v $
 * Revision 1.1.4.2  2007/01/25 16:11:35  jorpiell
 * Se han cambiado los imports que hacan referencia a remoteServices. Esto es as, porque se han renombrado las clases del driver de GML
 *
 * Revision 1.4  2007/01/08 12:12:03  csanchez
 * Corregida visualización de Tablas simples
 *
 * Revision 1.3  2006/12/29 17:12:10  jorpiell
 * Se tienen en cuenta los simpleTypes y los choices, adems de los atributos multiples
 *
 * Revision 1.2  2006/10/10 12:55:27  jorpiell
 * Se ha aadido el soporte de features complejas
 *
 * Revision 1.1  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FMAPGeometryFactory implements IGeometriesFactory{
	private static Logger logger = Logger.getLogger(FMAPGeometryFactory.class);
	
	public Object createPoint2D(double x, double y){
		return ShapeFactory.createPoint2D(x,y);
	}

	public Object createPoint2D(Point2D point2D) {
		return createPoint2D(point2D.getX(),point2D.getY());
	}
	
	public Object createMultipoint2D(double[] x, double[] y){
		return ShapeFactory.createMultipoint2D(x,y);
	}
	
	
	public Object createSimpleFeature(String element, IXMLType type,Map values,Object geometry) {
		Object[] fmapParams = values.keySet().toArray();
		Value[] fmapValues = getValues(values);
		IFeature feature = new DefaultFeature((IGeometry)geometry,fmapValues);
		return new FeatureWithAttributes(feature,fmapParams,fmapValues);
	}

	public Object createGeometry(String gmlGeometry) {
		GMLReader reader = new GMLReader();
		try {
			Geometry geom = reader.read(gmlGeometry,new GeometryFactory());
			return ShapeFactory.createGeometry(FConverter.jts_to_java2d(geom));
		} catch (Exception e){
			logger.error("Can't parse: " + gmlGeometry,e);
		}
		return null;
	}
	
	private Value[] getValues(Map values){
		int i = 0;	
		Set keys = values.keySet();
		Iterator it = keys.iterator();
		Value[] returnValues = new Value[keys.size()];
		while(it.hasNext()){
			String key = (String)it.next();
			Object obj = values.get(key);
			/**************	
		   	 * <MULTIPLE> * 
		   	 **************/	
			if (obj instanceof ArrayList){
				returnValues[i] = getMultipleValue(key,(ArrayList)obj);
			}
			/*************	
		   	 * <COMPLEX> *  
		   	 *************/	
			else if(obj instanceof LinkedHashMap){
				returnValues[i]  = getComplexValue(key,(LinkedHashMap)obj);
			}
			/************	
		   	 * <SIMPLE> *  
		   	 ************/
			else{
				returnValues[i] = getValue(obj);
			}	
			i++;
		}
		return returnValues;
	}
	
	private Value getComplexValue(String name,Map hash){
		ComplexValue value = ValueFactory.createComplexValue(name);
		Set keys = hash.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()){
			String key = (String)it.next();
			Object obj = hash.get(key);		
			if (obj instanceof ArrayList){
				value.put(key,getMultipleValue(key,(ArrayList)obj));
			}else if(obj instanceof LinkedHashMap){
				value.put(key,getComplexValue(key,(LinkedHashMap)obj));
			}else{
				value.put(key,getValue(obj));
			}
		}
		return value;
	}
	
	private Value getMultipleValue(String name,ArrayList al){
		if (al.size()==0){
			return null;
		}
		else if ((al.size()==1)&&(!((al.get(0) instanceof ArrayList)||(al.get(0) instanceof LinkedHashMap)))){
			return getValue(al.get(0));
		}
		else{
			ComplexValue value = ValueFactory.createComplexValue(name);
			for (int i=0 ; i<al.size() ; i++){
				String attName = name + "_" + i;
				Object obj = al.get(i);
				/**************	
			   	 * <MULTIPLE> * 
			   	 **************/	
				if (obj instanceof ArrayList){
					value.put(attName,getMultipleValue(attName,(ArrayList)obj));
				}
				/*************	
			   	 * <COMPLEX> *  
			   	 *************/	
				else if(obj instanceof LinkedHashMap){
					value.put(attName,getComplexValue(attName,(LinkedHashMap)obj));
				}
				/************	
			   	 * <SIMPLE> *  
			   	 ************/	
				else{
					value.put(attName,getValue(obj));
				}
			}
			return value;
		}
	}
	
	/**
	 * Gets the attributes
	 * @param attr
	 * @return
	 */
	private Value getValue(Object attr) {	
		if (attr instanceof Double){
			return ValueFactory.createValue(((Double)attr).doubleValue());
		}else if (attr instanceof String){
			return ValueFactory.createValue(String.valueOf(attr));
		}else if (attr instanceof Long){
			return ValueFactory.createValue(((Long)attr).longValue());
		}else if (attr instanceof Integer){
			return ValueFactory.createValue(((Integer)attr).intValue());
		}else if (attr instanceof Float){
			return ValueFactory.createValue(((Float)attr).floatValue());
		}else if (attr instanceof Short){
			return ValueFactory.createValue(((Short)attr).shortValue());
		}else if (attr instanceof Boolean){
			return ValueFactory.createValue(((Boolean)attr).booleanValue());
		}else if (attr instanceof Date){
			return ValueFactory.createValue(((Date)attr));
		}
		return ValueFactory.createValue(((String)""));
	}
	
	public class FeatureWithAttributes{
		private IFeature feature;
		private Object[] attributeName;
		private Value[] attributeValue;
		
		public FeatureWithAttributes(IFeature feature, Object[] attributeName, Value[] attributeValue) {
			super();		
			this.feature = feature;
			this.attributeName = attributeName;
			this.attributeValue = attributeValue;
		}

		/**
		 * @return Returns the attributeName.
		 */
		public Object[] getAttributeName() {
			return attributeName;
		}

		/**
		 * @return Returns the feature.
		 */
		public IFeature getFeature() {
			return feature;
		}

		/**
		 * @return Returns the attributeValue.
		 */
		public Value[] getAttributeValue() {
			return attributeValue;
		}
		
	}


}
