/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;
import com.vividsolutions.jts.geom.Envelope;

import java.awt.geom.Rectangle2D;
import java.util.List;


/**
 * Interfaz usada internamente para acceder a fuentes de datos vectoriales en
 * modo lectura
 */
public interface ReadableVectorial {
	/**
	 * Indica que se va a comenzar a hacer una serie de operaciones sobre el
	 * origen de datos con la finalidad de que dicho origen de datos se
	 * prepare (abra el fichero, comience una transaccin, conecte al
	 * servidor, ...). Abre el fichero de ndices en caso de que exista.
	 *
	 * @throws DriverIOException
	 */
	void start() throws DriverIOException;

	/**
	 * Hace que se cierre el soporte fsico de la capa. Cierra el fichero de
	 * ndices en caso de que exista.
	 *
	 * @throws DriverIOException
	 */
	void stop() throws DriverIOException;

	/**
	 * Devuelve la geometra a partir de un ndice.
	 *
	 * @param index ndice.
	 *
	 * @return Geometra.
	 *
	 * @throws DriverIOException
	 */
	IGeometry getShape(int index) throws DriverIOException;

	/**
	 * Devuelve el nmero de Shape.
	 *
	 * @return Nmero de Shape.
	 *
	 * @throws DriverIOException
	 */
	int getShapeCount() throws DriverIOException;

	/**
	 * Devuelve la extensin total de la capa.
	 *
	 * @return Extensin total.
	 *
	 * @throws DriverIOException
	 */
	Rectangle2D getFullExtent() throws DriverIOException;

	/**
	 * Obtiene el tipo de las geometras almacenadas en esta fuente de datos
	 *
	 * @return Obtiene el tipo de la capa. Es un bit-oring de los tipos
	 * 		   definidos en FShape POINT, LINE, POLYGON o TEXT;
	 *
	 * @throws DriverIOException
	 */
	int getShapeType() throws DriverIOException;
	/**
	 * Obtiene una referencia al objeto que implementa la interfaz vectorial
	 * con el fin de que las Strategy puedan optimizar en funcin del driver.
	 *
	 * @return VectorialDriver
	 */
	public VectorialDriver getDriver();
	/**
	 * Establece el driver sobre el que acta el adaptador
	 *
	 * @param driver
	 */
	public void setDriver(VectorialDriver driver);
	 /*
     * (non-Javadoc)
     *
     * @see com.iver.cit.gvsig.fmap.layers.VectorialAdapter#getRecordset()
     */
    public SelectableDataSource getRecordset() throws DriverLoadException;
    public DriverAttributes getDriverAttributes();

    /**
     * En la implementacin por defecto podemos hacer que cada
     * feature tenga ID = numero de registro.
     * En el DBAdapter podramos "overrride" este mtodo y poner
     * como ID de la Feature el campo nico escogido en
     * la base de datos. Bsicamente es por comodidad.
     * @param numReg
     * @return
     * @throws DriverException
     */
    public IFeature getFeature(int numReg) throws DriverException;
    
    /* public IFeatureIterator getFeatureIterator(String strEPSG) throws DriverException;
    public IFeatureIterator getFeatureIterator(Rectangle2D r, String strEPSG) throws DriverException; */

	//void setRecordset(SelectableDataSource sds);



}
