/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.tools;

import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.tools.Behavior.Behavior;
import com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener;

import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

import java.util.ArrayList;
import java.util.Iterator;


/**
 * Behavior mediante el que se pueden combinar varios behaviors
 * a la vez en un mismo MapControl
 *
 * @author Fernando Gonzlez Corts
 */
public class CompoundBehavior extends Behavior {
	private ArrayList mapTools = new ArrayList();
	private ArrayList draws = new ArrayList();

	public CompoundBehavior(Behavior[] tools){
		for (int i = 0; i < tools.length; i++) {
			mapTools.add(tools[i]);
			if (i == 0)
				draws.add(Boolean.TRUE);
			else
				draws.add(Boolean.FALSE);
		}
	}
	
	public void addMapBehavior(Behavior mt, boolean draw){
		mapTools.add(mt);
		draws.add(new Boolean(draw));
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Cursor getCursor() {
		if (mapTools.size() > 0) {
			return ((Behavior) mapTools.get(0)).getCursor();
		} else {
			return null;
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseClicked(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseClicked(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseDragged(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseDragged(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseEntered(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseEntered(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseExited(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseExited(e);
		}
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseMoved(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseMoved(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mousePressed(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mousePressed(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseReleased(MouseEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseReleased(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @throws BehaviorException DOCUMENT ME!
	 */
	public void mouseWheelMoved(MouseWheelEvent e) throws BehaviorException {
		for (Iterator iter = mapTools.iterator(); iter.hasNext();) {
			Behavior mapTool = (Behavior) iter.next();
			
			mapTool.mouseWheelMoved(e);
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param g DOCUMENT ME!
	 */
	public void paintComponent(Graphics g) {
		for (int i = 0; i < mapTools.size(); i++) {
			Behavior mapTool = (Behavior) mapTools.get(i);
			
			if (((Boolean) draws.get(i)).booleanValue())
				mapTool.paintComponent(g);
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#setListener(com.iver.cit.gvsig.fmap.tools.Listeners.ToolListener)
	 */
	public void setListener(ToolListener listener) {
		if (listener != null) {
			throw new RuntimeException(
				"CompoundBehavior does not have listeners");
		}
	}

	/**
	 * @see com.iver.cit.gvsig.fmap.tools.Behavior.Behavior#getListener()
	 */
	public ToolListener getListener() {
		return null;
	}
	public void setMapControl(MapControl mc) {
		for (int i = 0; i < mapTools.size(); i++) {
			Behavior mapTool = (Behavior) mapTools.get(i);
			
			mapTool.setMapControl(mc);
		}
		
		super.setMapControl(mc);
	}
}
