/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs;

import java.awt.dnd.Autoscroll;
import java.util.ArrayList;

/**
 * Clase que parsea una cadena wkt y consigue todos sus parmetros
 * de manera independiente para poder ser utilizados
 * en distintas funciones
 * 
 * @author Jos Luis Gmez Martnez
 *
 */

public class CrsWkt {
	private String projcs = "";
	private String geogcs = "";
	private String datum = "";
	private String[] spheroid = {"" ,"",""};
	private String[] primem = {"",""};
	private String[] unit = {"",""};
	private String[] unit_p = {"",""};
	private String projection = "";
	private String[] param_name;
	private String[] param_value;
	private int contador = 0;
	private String[] authority = {"",""};
	
	public CrsWkt(String wkt_spaces) {
		String aux;
		String wkt = "";
		for(int i = 0; i < wkt_spaces.length(); i++) {
			aux = ""+wkt_spaces.charAt(i);
			if(!aux.equals(" ")) {
				wkt+=aux;
			}else {
				wkt += "";
			}
		}
		fromWKT(wkt, false);
	}
	
	/**
	 * Acepta la cadena wkt y si es un CRS proyectado para conseguir el parser
	 * de los dos modelos de cadena wkt que se utilizan, una cuando es base
	 * y otra cuando es proyectado
	 * @param wkt
	 * @param isProj
	 */
	private void fromWKT(String wkt, boolean isProj) {
		
		String res = new String();
		if(!wkt.startsWith("EPSG:")) {
			res = "";
			for(; wkt.charAt(contador) != '"'; contador++) {
				res += wkt.charAt(contador);
			}
			if(res.equals("GEOGCS[")) {
				
				contador++;
				for(; wkt.charAt(contador) != '"'; contador++) {
					geogcs += wkt.charAt(contador);
				}
				
				res = "";
				contador++;
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",DATUM[")) {
					contador++;
					for(; wkt.charAt(contador) != '"'; contador++) {
						datum += wkt.charAt(contador);
					}
				}
				res = "";
				contador++;
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",SPHEROID[")) {
					contador++;
					for(; wkt.charAt(contador) != ']'; contador++) {
						while(wkt.charAt(contador) != '"' ) {
							spheroid[0] += wkt.charAt(contador);
							contador++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ',') {
							spheroid[1] += wkt.charAt(contador);
							contador++;
						}
						contador++;
						while(wkt.charAt(contador) != ']') {
							spheroid[2] += wkt.charAt(contador);
							contador++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",PRIMEM[")) {
					contador++;
					for(; wkt.charAt(contador) != ','; contador++) {
						while(wkt.charAt(contador) != '"' ){
							primem[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							primem[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals("UNIT[")) {
					contador++;
					for(; wkt.charAt(contador) != ','; contador++) {
						while(wkt.charAt(contador) != '"' ){
							unit[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							unit[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals("AUTHORITY[")) {
					contador++;
					for(; wkt.charAt(contador) != ']'; contador++) {
						while(wkt.charAt(contador) != '"' ){
							authority[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							authority[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
			}else if (res.equals("PROJCS[")) {
				contador++;
				for(; wkt.charAt(contador) != '"'; contador++) {
					projcs += wkt.charAt(contador);
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",GEOGCS[")) {
					contador++;
					for(; wkt.charAt(contador) != '"'; contador++) {
						geogcs += wkt.charAt(contador);
					}
				}
								
				res = "";
				contador++;
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",DATUM[")) {
					contador++;
					for(; wkt.charAt(contador) != '"'; contador++) {
						datum += wkt.charAt(contador);
					}
				}
				res = "";
				contador++;
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",SPHEROID[")) {
					contador++;
					for(; wkt.charAt(contador) != ']'; contador++) {
						while(wkt.charAt(contador) != '"' ) {
							spheroid[0] += wkt.charAt(contador);
							contador++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ',') {
							spheroid[1] += wkt.charAt(contador);
							contador++;
						}
						contador++;
						while(wkt.charAt(contador) != ']') {
							spheroid[2] += wkt.charAt(contador);
							contador++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",PRIMEM[")) {
					contador++;
					for(; wkt.charAt(contador) != ','; contador++) {
						while(wkt.charAt(contador) != '"' ){
							primem[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							primem[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals("UNIT[")) {
					contador++;
					for(; wkt.charAt(contador) != ']'; contador++) {
						while(wkt.charAt(contador) != '"' ){
							unit[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							unit[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals(",PROJECTION[")) {
					contador++;
					for(; wkt.charAt(contador) != '"'; contador++) {
						projection += wkt.charAt(contador);
					}
				}
				contador = contador+2;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				//Hallamos el numero de parametros que tiene la cadena wkt
				int i = 0;
				
				int copiacontador = contador;
				if(res.equals(",PARAMETER[")) {
					do{
						for(; wkt.charAt(copiacontador) != ']'; copiacontador++) {
							while(wkt.charAt(copiacontador) != '"' )
								copiacontador++;
							copiacontador += 2;
							while(wkt.charAt(copiacontador) != ']' )
								copiacontador++;
							copiacontador--;
						}
						i++;
						copiacontador++;
						res = "";
						for(; wkt.charAt(copiacontador) != '"'; copiacontador++) {
							res += wkt.charAt(copiacontador);
						}
					} while (res.equals(",PARAMETER["));
					res = ",PARAMETER[";
				}
				// Inicializamos los parametros
				param_name = new String[i];
				param_value = new String[i];
				for(int j = 0 ;j < i; j++ ){
					param_name[j] = "";
					param_value[j] = "";
				}
				i = 0;
				if(res.equals(",PARAMETER[")) {
					do{
						contador++;
						for(; wkt.charAt(contador) != ']'; contador++) {
							while(wkt.charAt(contador) != '"' ){
								param_name[i] += wkt.charAt(contador);
								contador++;
							}
							contador += 2;
							while(wkt.charAt(contador) != ']' ){
								param_value[i] += wkt.charAt(contador);
								contador++;
							}
							contador--;
						}
						i++;
						contador++;
						res = "";
						for(; wkt.charAt(contador) != '"'; contador++) {
							res += wkt.charAt(contador);
						}
					} while (res.equals(",PARAMETER["));					
				}
				
				if (res.equals(",UNIT[")){
					contador++;
					for(; wkt.charAt(contador) != ','; contador++) {
						while(wkt.charAt(contador) != '"' ){
							unit_p[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							unit_p[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
				contador++;
				res = "";
				for(; wkt.charAt(contador) != '"'; contador++) {
					res += wkt.charAt(contador);
				}
				if(res.equals("AUTHORITY[")) {
					contador++;
					for(; wkt.charAt(contador) != ']'; contador++) {
						while(wkt.charAt(contador) != '"' ){
							authority[0] += wkt.charAt(contador);
							contador ++;
						}
						contador +=2;
						while(wkt.charAt(contador) != ']') {
							authority[1] += wkt.charAt(contador);
							contador ++;
						}
					}
				}
			}
			else if (res.equals("GEOCCS[")){
				/*
				 * parte necesaria para capturar la cadena geocentrica...
				 */
				
			}
		}else
			geogcs = wkt;
	}
	
	public String getProjection() {
		return projection;
	}
	
	public String getProjcs() {
		return projcs;
	}
	
	public String getGeogcs() {
		return geogcs;
	}
	
	public String getDatumName() {
		return datum;
	}
	
	public String[] getSpheroid() {
		return spheroid;
	}
	
	public String[] getPrimen() {
		return primem;
	}
	
	public String getName() {
		if(projcs == "") 
			return geogcs;
		return projcs;
	}
	
	public String[] getUnit() {
		return unit;
	}
	
	public String[] getUnit_p() {
		return unit_p;
	}
	
	public String[] getParam_name() {
		return param_name;
	}
	
	public String[] getParam_value() {
		return param_value;
	}
	
	public String[] getAuthority(){
		return authority;
	}
	
	/*
	 * Parser a medio hacer, orientado a objeto y recursivo.
	 * by LWS.
	 */
	
	public static class WKT {
		public class Param {
			String key;
			ArrayList values = new ArrayList();
			ArrayList params = new ArrayList();
			public int pos = 0;
			public Param(String key) {
				this.key = key;
			}
			public void addValue(String name) {
				values.add(name);
			}
			public void addParam(Param p) {
				params.add(p);
			}
		}
		String data;
		public WKT(String data) {
			this.data = data;
			WKT.Param param = parseParam(0);
		}
		private WKT.Param parseParam(int pos) {
			WKT.Param param = null;
			String key, name;
			
			int l = data.length();
			for (int i = pos; i<l;) {
				int nextParam = data.indexOf(",", i);
				if (nextParam == i) {
					nextParam = data.indexOf(",", ++i);
				}
				int cierra = data.indexOf("]", i);
				int abre = data.indexOf("[", i);
				if (cierra < abre) { // Esta mal
					pinta(" =>");
					param.pos = cierra;
					return param;
				}
				if (param == null) {
					if (abre > 0) { //hay claves
						key = data.substring(i, data.indexOf("[", i));
						pinta(key+ " <= ");
						i = abre+1;
						param = new WKT.Param(key);
					}
				} else {
					if (data.substring(i).startsWith("\"")) {
						name = data.substring(i+1, data.indexOf("\"", i+1));
						i+=name.length()+2;
						pinta("|"+name+"|,");
						param.addValue(name);
					} else if (nextParam<abre) {
						name = data.substring(i, data.indexOf(",", i));
						i+=name.length();
						pinta(name+",");
						param.addValue(name);
					} else {
						Param p = parseParam(i);
						i = p.pos+1;
					}
				}
			}
			return param;
		}
		static int cnt=0;
		public static void pinta(String str) {
			cnt++;
			if (cnt>60)
				System.exit(1);
			System.out.println(str);
		}
	}
	
	private void parseWKT(String data) {
		WKT wkt = new WKT(data);
	}
}