/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 Instituto de Desarrollo Regional and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   Instituto de Desarrollo Regional (Universidad de Castilla La-Mancha)
 *   Campus Universitario s/n
 *   02071 Alabacete
 *   Spain
 *
 *   +34 967 599 200
 */

package org.gvsig.crs.gui.panels;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;

import org.cresques.ui.DefaultDialogPanel;
import org.gvsig.crs.ICrs;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;

/**
 * Clase que genera el panel de informacin del CRS seleccionado
 * en el caso del repositorio de la EPSG
 * @author Miguel Garca Jimnez (garciajimenez.miguel@gmail.com)
 *
 */
public class InfoCRSPanel extends DefaultDialogPanel implements IWindow, ActionListener{
	
	private static final long serialVersionUID = 1L;

	private JPanel panelLabels;
	
	private JTable jTable;
	public DefaultTableModel dtm = null;
	private JScrollPane jScrollPane1 = null;
	private JPanel jPanelbuttons;
	private JButton jButtonOk;
	
	private ICrs proj;
	
	private JLabel jLabelProjcs;
	private JLabel jLabelGeogcs;
	private JLabel jLabelDatum;
	private JLabel jLabelSpheroid;
	private JLabel jLabelPrimem;
	private JLabel jLabelProjection;
	private JLabel jLabelUnits;
	private JLabel jLabelUnits_p;
	private JLabel jLabelProjcsdinamic;
	private JLabel jLabelGeogcsdinamic;
	private JLabel jLabelDatumdinamic;
	private JLabel jLabelSpheroiddinamic;
	private JLabel jLabelPrimemdinamic;
	private JLabel jLabelProjectiondinamic;
	private JLabel jLabelUnitsdinamic;
	private JLabel jLabelUnits_pdinamic;
	
	public InfoCRSPanel(ICrs p) {
		super(false);
		proj = p;	
		inicializate();
	}
	
	private void inicializate() {
		this.add(getPanel(), null);
		this.add(getJPanelButtons(),null);
	}

	private JPanel getPanel() {
		if(panelLabels == null) {
			panelLabels = new JPanel();
			panelLabels.setLayout(new GridLayout(6,4));
			panelLabels.setLayout(new FlowLayout(FlowLayout.LEADING,5,10));
			panelLabels.setPreferredSize(new Dimension(525, 300));
			panelLabels.add(getJLabelProjcs(),null);
			panelLabels.add(getJLabelProjcsDinamic(),null);
			panelLabels.add(getJLabelGeogcs(),null);
			panelLabels.add(getJLabelGeogcsDinamic(),null);
			panelLabels.add(getJLabelDatum(),null);
			panelLabels.add(getJLabelDatumDinamic(),null);
			panelLabels.add(getJLabelSpheroid(),null);
			panelLabels.add(getJLabelSpheroidDinamic(),null);
			panelLabels.add(getJLabelPrimen(),null);
			panelLabels.add(getJLabelPrimenDinamic(),null);
			panelLabels.add(getJLabelUnits(),null);
			panelLabels.add(getJLabelUnitsDinamic(),null);
			panelLabels.add(getJLabelProjection(),null);
			panelLabels.add(getJLabelProjectionDinamic(),null);
			panelLabels.add(getJLabelUnits_p(),null);
			panelLabels.add(getJLabelUnits_pDinamic(),null);
			panelLabels.add(getJScrollPane1(), null);
		}
		return panelLabels;
	}
	
	private JPanel getJPanelButtons() {
		if(jPanelbuttons == null) {
			jPanelbuttons = new JPanel();
			jPanelbuttons.setLayout(new FlowLayout(FlowLayout.RIGHT));
			jPanelbuttons.setPreferredSize(new Dimension(525,50));
			jPanelbuttons.add(getJButtonOk(),null);
		}
		return jPanelbuttons;
	}
	
	private JButton getJButtonOk() {
		if(jButtonOk == null) {
			jButtonOk = new JButton();
			jButtonOk.setText(PluginServices.getText(this,"ok"));
			jButtonOk.setPreferredSize(new Dimension(100,25));
			jButtonOk.setMnemonic('O');
			jButtonOk.setToolTipText("Accept");
			jButtonOk.addActionListener(this);
		}
		return jButtonOk;
	}
	
	private JLabel getJLabelProjcs() {
		if(jLabelProjcs == null) {
			jLabelProjcs = new JLabel();
			jLabelProjcs.setPreferredSize(new Dimension(75,20));
			jLabelProjcs.setFont(new Font("Projcs:",Font.BOLD,10));
			jLabelProjcs.setText(PluginServices.getText(this,"projcs")+":");
		}
		return jLabelProjcs;
	}
	
	private JLabel getJLabelProjcsDinamic() {
		if(jLabelProjcsdinamic == null) {
			jLabelProjcsdinamic = new JLabel();
			jLabelProjcsdinamic.setPreferredSize(new Dimension(150,20));
			jLabelProjcsdinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelProjcsdinamic.setText(proj.getCrsWkt().getProjcs());
		}
		return jLabelProjcsdinamic;
	}
	
	private JLabel getJLabelGeogcs() {
		if(jLabelGeogcs == null) {
			jLabelGeogcs = new JLabel();
			jLabelGeogcs.setPreferredSize(new Dimension(75,20));
			jLabelGeogcs.setFont(new Font("Geogcs:",Font.BOLD,10));
			jLabelGeogcs.setText(PluginServices.getText(this,"geogcs")+ ":");
		}
		return jLabelGeogcs;
	}
	
	private JLabel getJLabelGeogcsDinamic() {
		if(jLabelGeogcsdinamic == null) {
			jLabelGeogcsdinamic = new JLabel();
			jLabelGeogcsdinamic.setPreferredSize(new Dimension(150,20));
			jLabelGeogcsdinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelGeogcsdinamic.setText(proj.getCrsWkt().getGeogcs());
		}
		return jLabelGeogcsdinamic;
	}
	
	private JLabel getJLabelDatum() {
		if(jLabelDatum == null) {
			jLabelDatum = new JLabel();
			jLabelDatum.setPreferredSize(new Dimension(75,20));
			jLabelDatum.setFont(new Font("Datum:",Font.BOLD,10));
			jLabelDatum.setText(PluginServices.getText(this,"datum")+":");
		}
		return jLabelDatum;
	}
	
	private JLabel getJLabelDatumDinamic() {
		if(jLabelDatumdinamic == null) {
			jLabelDatumdinamic = new JLabel();
			jLabelDatumdinamic.setPreferredSize(new Dimension(150,20));
			jLabelDatumdinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelDatumdinamic.setText(proj.getCrsWkt().getDatumName());
		}
		return jLabelDatumdinamic;
	}
	
	private JLabel getJLabelSpheroid() {
		if(jLabelSpheroid == null) {
			jLabelSpheroid = new JLabel();
			jLabelSpheroid.setPreferredSize(new Dimension(75,20));
			jLabelSpheroid.setFont(new Font("Spheroid",Font.BOLD,10));
			jLabelSpheroid.setText(PluginServices.getText(this,"spheriod")+":");
		}
		return jLabelSpheroid;
	}
	
	private JLabel getJLabelSpheroidDinamic() {
		if(jLabelSpheroiddinamic == null) {
			String[] sphe = proj.getCrsWkt().getSpheroid();
			jLabelSpheroiddinamic = new JLabel();
			jLabelSpheroiddinamic.setPreferredSize(new Dimension(200,20));
			jLabelSpheroiddinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelSpheroiddinamic.setText(sphe[0]+" , "+sphe[1]+" , "+sphe[2]);
		}
		return jLabelSpheroiddinamic;
	}
	
	private JLabel getJLabelPrimen() {
		if(jLabelPrimem == null) {
			jLabelPrimem = new JLabel();
			jLabelPrimem.setPreferredSize(new Dimension(75,20));
			jLabelPrimem.setFont(new Font("Primen:",Font.BOLD,10));
			jLabelPrimem.setText(PluginServices.getText(this,"primem")+":");
		}
		return jLabelPrimem;
	}
	
	private JLabel getJLabelPrimenDinamic() {
		if(jLabelPrimemdinamic == null) {
			String pri[] = proj.getCrsWkt().getPrimen();
			jLabelPrimemdinamic = new JLabel();
			jLabelPrimemdinamic.setPreferredSize(new Dimension(150,20));
			jLabelPrimemdinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelPrimemdinamic.setText(pri[0]+" , "+pri[1]);
		}
		return jLabelPrimemdinamic;
	}
	
	private JLabel getJLabelProjection() {
		if(jLabelProjection == null) {
			jLabelProjection = new JLabel();
			jLabelProjection.setPreferredSize(new Dimension(75,20));
			jLabelProjection.setFont(new Font("Projection",Font.BOLD,10));
			jLabelProjection.setText(PluginServices.getText(this,"projection")+":");
		}
		return jLabelProjection;
	}
	
	private JLabel getJLabelProjectionDinamic() {
		if(jLabelProjectiondinamic == null) {
			jLabelProjectiondinamic = new JLabel();
			jLabelProjectiondinamic.setPreferredSize(new Dimension(150,20));
			jLabelProjectiondinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelProjectiondinamic.setText(proj.getCrsWkt().getProjection());
		}
		return jLabelProjectiondinamic;
	}
	
	private JLabel getJLabelUnits() {
		if(jLabelUnits == null) {
			jLabelUnits = new JLabel();
			jLabelUnits.setPreferredSize(new Dimension(75,20));
			jLabelUnits.setFont(new Font("Units",Font.BOLD,10));
			jLabelUnits.setText(PluginServices.getText(this,"units")+":");
		}
		return jLabelUnits;
	}
	
	private JLabel getJLabelUnitsDinamic() {
		if(jLabelUnitsdinamic == null) {
			String[] units = proj.getCrsWkt().getUnit();
			jLabelUnitsdinamic = new JLabel();
			jLabelUnitsdinamic.setPreferredSize(new Dimension(200,20));
			jLabelUnitsdinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelUnitsdinamic.setText(units[0]+" , "+units[1]);
		}
		return jLabelUnitsdinamic;
	}
	
	private JLabel getJLabelUnits_p() {
		if(jLabelUnits_p == null) {
			jLabelUnits_p = new JLabel();
			jLabelUnits_p.setPreferredSize(new Dimension(75,20));
			jLabelUnits_p.setFont(new Font("",Font.BOLD,10));
			jLabelUnits_p.setText(PluginServices.getText(this,"units_p")+":");
		}
		return jLabelUnits_p;
	}
	
	private JLabel getJLabelUnits_pDinamic() {
		if(jLabelUnits_pdinamic == null) {
			String[] uni_p = proj.getCrsWkt().getUnit_p();
			jLabelUnits_pdinamic = new JLabel();
			jLabelUnits_pdinamic.setPreferredSize(new Dimension(150,20));
			jLabelUnits_pdinamic.setFont(new Font("",Font.ROMAN_BASELINE,10));
			jLabelUnits_pdinamic.setText(uni_p[0]+" , "+uni_p[1]);
			
		}
		return jLabelUnits_pdinamic;
	}
	
	private JScrollPane getJScrollPane1() {
		if(jScrollPane1 == null) {
			jScrollPane1 = new JScrollPane();
			jScrollPane1.setPreferredSize(new Dimension(400,150));
		/*	jScrollPane1.setBorder(
				    BorderFactory.createCompoundBorder(
					BorderFactory.createCompoundBorder(
							BorderFactory.createTitledBorder("Transformations"),
							BorderFactory.createEmptyBorder(5,5,5,5)),
							jScrollPane1.getBorder()));*/
			jScrollPane1.setViewportView(getJTable());
		}
		return jScrollPane1;
	}
	
	private JTable getJTable() {
		if(jTable == null) {
			String[] columnNames = {PluginServices.getText(this,"nombre")
					,PluginServices.getText(this,"valor")};
			String[] param_n = proj.getCrsWkt().getParam_name();
			String[] param_v = proj.getCrsWkt().getParam_value();
			Object[][]data;
			if(param_v != null) {
				data = new Object[param_v.length][2];
				for(int i = 0 ; i < 2 ; i++ )
					for(int j = 0 ; j < param_n.length ; j++) {
						if(i == 0)
							data[j][i] = param_n[j];
						else
							data[j][i] = param_v[j];
					}
			} else 
				data = new Object[1][1];
			dtm = new DefaultTableModel(data, columnNames);
			jTable = new JTable(dtm);
			jTable.setCellSelectionEnabled(false);
			jTable.setRowSelectionAllowed(true);
			jTable.setColumnSelectionAllowed(false);
			jTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			
		}
		return jTable;
	}

	public WindowInfo getWindowInfo() {
		WindowInfo m_viewinfo=new WindowInfo(WindowInfo.MODALDIALOG);
   		m_viewinfo.setTitle(PluginServices.getText(this,proj.getCrsWkt().getName()));
		return m_viewinfo;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == getJButtonOk()){
			PluginServices.getMDIManager().closeWindow(this);
		}
	}
}
