package org.gvsig.remoteClient.gml.v2;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Vector;

import org.gvsig.remoteClient.gml.GMLTags;
import org.gvsig.remoteClient.gml.IGMLFeaturesIterator;
import org.gvsig.remoteClient.gml.exceptions.BaseException;
import org.gvsig.remoteClient.gml.exceptions.GMLException;
import org.gvsig.remoteClient.gml.exceptions.GMLFileReadException;
import org.gvsig.remoteClient.gml.exceptions.GMLParserException;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.factories.XMLElementsFactory;
import org.gvsig.remoteClient.gml.schemas.XMLElement;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.gml.types.GMLGeometries;
import org.gvsig.remoteClient.gml.types.IXMLType;
import org.gvsig.remoteClient.gml.types.XMLComplexType;
import org.gvsig.remoteClient.gml.types.XMLSimpleType;
import org.gvsig.remoteClient.gml.utils.GMLUtilsParser;
import org.gvsig.remoteClient.gml.warnings.GMLWarningInfo;
import org.gvsig.remoteClient.gml.warnings.GMLWarningWrongNamespace;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFeaturesIterator_v2.java,v 1.1.2.7 2007/06/19 12:19:49 jorpiell Exp $
 * $Log: GMLFeaturesIterator_v2.java,v $
 * Revision 1.1.2.7  2007/06/19 12:19:49  jorpiell
 * Add a list of deafult attributes
 *
 * Revision 1.1.2.6  2007/03/23 10:42:30  jorpiell
 * Aadido soporte para elementos anidadas con el mismo nombre que el elemento padre
 *
 * Revision 1.1.2.5  2007/01/25 16:12:59  jorpiell
 * Se han sustituido las clases por las que hay en el nuevo driver de GML.
 *
 * Revision 1.9  2007/01/15 13:11:17  csanchez
 * Sistema de Warnings y Excepciones adaptado a BasicException
 *
 * Revision 1.8  2006/12/29 18:05:20  jorpiell
 * Se tienen en cuenta los simpleTypes y los choices, adems de los atributos multiples
 *
 * Revision 1.6  2006/12/22 11:25:44  csanchez
 * Nuevo parser GML 2.x para gml's sin esquema
 *
 * Revision 1.5  2006/11/06 12:14:38  jorpiell
 * Cuando geottols no es capaz de parsear el gML se cargaba la capa con el "aspa roja". Ahora esto ya no ocurre, porque si geotools no es capaz de parsear el gML la geometra se descarta
 *
 * Revision 1.4  2006/10/10 12:52:28  jorpiell
 * Soporte para features complejas.
 *
 * Revision 1.3  2006/10/02 08:33:49  jorpiell
 * Cambios del 10 copiados al head
 *
 * Revision 1.1.2.2  2006/09/25 11:35:15  jorpiell
 * Se tienen en cuanta tablas a distintos niveles. En caso de anidamiento se cogen los resultados de la primera tabla que aparezca.
 *
 * Revision 1.1.2.1  2006/09/19 12:23:15  jorpiell
 * Ya no se depende de geotools
 *
 * Revision 1.2  2006/09/18 12:08:55  jorpiell
 * Se han hecho algunas modificaciones que necesitaba el WFS
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 */
public class GMLFeaturesIterator_v2 extends IGMLFeaturesIterator {
	public GMLWarningInfo warnings = null;
	private boolean goNextTag= true;
	
	public GMLFeaturesIterator_v2(XMLSchemaParser parser, IGeometriesFactory factory) {
		super(parser, factory);	
		//It creates the warnings structure
		warnings = new GMLWarningInfo();
	}
	
	/**
	 * hasNext()
	 * @return boolean
	 * 			-true: if has more features to parse
	 * 			-false: in other case.  
	 **/
	public boolean hasNext() throws GMLException {
		try {
			if ((getParser().getName().compareTo(getFeatureRoot()) == 0) &&
				(getParser().getEventType() == KXmlParser.START_TAG)){
				return true;
			}
		} catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLException(new GMLParserException(e));
		}		
		return false;
	}
	
	/**
	 * It implements the iterator function to parse one feature
	 * @return object (feature)
	 **/
	public Object next() throws GMLException{
		boolean endFeature = false;
		Object feature = null;
		int currentTag;		
		
		try {	
			// It returns what kind of tag is 
			currentTag = getParser().getEventType();	
			while (!endFeature){
				switch(currentTag){
				case KXmlParser.START_TAG:
					// It gets the name of the feature <FeatureMember>
					feature = parseFeature(getParser().getName());
					break;
				case KXmlParser.END_TAG:
					//It compares with the name of tag captured before
					//If is equal then is the end of the fature </FeatureMember>
					if (getParser().getName().compareTo(getFeatureRoot()) == 0){
						endFeature = true;	
						currentTag = getParser().nextTag();
					}
					break;
				case KXmlParser.TEXT:
					//Text Tags are ignored
					break;
				}
				if (!endFeature){					
					if (getParser().getName().compareTo(getFeatureRoot()) != 0){
						currentTag = getParser().next();
					}
					else
					{
						endFeature=true;
						currentTag = getParser().nextTag();
					}
				}
			}
		}catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLException(new GMLParserException(e));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new GMLException(new GMLFileReadException(e));
		} catch (BaseException e) {
			throw new GMLException(e);
		} 		
		return feature;
	}
	
	/**
	 * parseFeature(Element):
	 * Parse an specific feature by its name, if this feature isn't declared in the schema 
	 * or it doesn't exist, function tries to parse it without schema.
	 * 
	 * @param String elementName
	 **/
	private Object parseFeature(String elementName) throws BaseException{
		int currentTag;
		boolean end = false;
		LinkedHashMap values = new LinkedHashMap();		
		Object geometry = null;
		Object feature = null;
		XMLElement entity = null;
		
		try{		
			currentTag = getParser().next();
			String entityTag = getParser().getName();
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					entity = XMLElementsFactory.getElement(getParser().getName());
					if (entity != null){
						geometry = parseComplexFeature(entity,values,geometry);			
						//When the feature is parsed, it's time to create it
						if (geometry != null){
							feature = getFactory().createSimpleFeature(entity.getName(),entity.getEntityType(),values,geometry);
						}
						end = true;
					}
					//if the element doesn't exist, it tries to parse it without schema.
					else{
						geometry = parseComplexFeatureNoSchema(entityTag,values);			
						//The feature is created without schema
						if (geometry != null){
							feature = getFactory().createSimpleFeature("GMLFeature",null,values,geometry);
						}
						end = true;
					}
					break;
				case KXmlParser.END_TAG:
					if ((getParser().getName().compareTo(elementName) == 0)){
						end = true;
					}
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = getParser().next();
				}	
			}
		}catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLParserException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new GMLFileReadException(e);
		} 
		return feature;
	}
	
	/**
	 * parseComplexFeature(entity,parameters,values):
	 * Parses a complex type
	 * 
	 * @param complexTypeName
	 * @param params
	 * @param values
	 * @return geometry
	 * @throws GMLException 
	 */
	private Object parseComplexFeature(XMLElement entity,Map values, Object geometry) throws BaseException{
		int currentTag = KXmlParser.START_TAG;
		boolean end = false;		
		
		if (entity.getChildren() != null){
			Vector attributes = entity.getChildren();
			for (int i=0 ; i<attributes.size() ; i++){
				XMLElement element = (XMLElement)attributes.get(i);
				values.put(element.getName(), null);				
			}
		}
		
		XMLComplexType entityType = (XMLComplexType)entity.getEntityType();
		try{
			if(goNextTag)
				currentTag = getParser().nextTag();					
			while (!end){			
				switch(currentTag){
				case KXmlParser.START_TAG:
					if (getParser().getName().compareTo(GMLTags.GML_BOUNDEDBY)==0){
						GMLUtilsParser.parseBoundedBy(getParser());
					}else{
						String attName = getParser().getName();
						//The atributte get the entity Type (SIMPLE, COMPLEX or GEOMETRY)
						XMLElement attribute = entityType.getAttribute(attName);
						//It compares the tag with the GML Standard
						GMLGeometries gmlGeometry = new GMLGeometries(attName);
						if ((attribute != null)||(gmlGeometry.isGML()==true)){
							if ((gmlGeometry.isGML()==true)||(attribute.getEntityType() != null)){
								/************
								 * <SIMPLE> *
								 ************/
								if ((gmlGeometry.isGML()==false)&&(attribute.getEntityType().getType() == IXMLType.SIMPLE)){
									if (attribute.isMultiple()){
										ArrayList multiple = null;
										if (values.get(attribute.getName()) == null){
											multiple = new ArrayList();
										}else{
											multiple = (ArrayList)values.get(attribute.getName());
										}
										getParser().next();
										multiple.add(((XMLSimpleType)attribute.getEntityType()).getJavaType(getParser().getText()));
										values.put(attName,multiple);
									}else{
										getParser().next();
										values.put(attName,((XMLSimpleType)attribute.getEntityType()).getJavaType(getParser().getText()));
									}
									//Jump the end tag
									getParser().next();
								}
								/**************
								 * <GEOMETRY> *
								 **************/
								else if ((gmlGeometry.isGML()==true)||(attribute.getEntityType().getType() == IXMLType.GML_GEOMETRY)){
									//if is GML geometry we try to parse it
									if (gmlGeometry.isGeometryGML()==true){
										geometry = GMLUtilsParser.parseGeometry(getParser(),attName,getFactory());
									}
									else{
										if (gmlGeometry.isFeatureGML()==false){
											//If isn't a known GML geometry, then is a geometry specified in the schema
											//we know that this is special tag of geometry and the real geometry is after it
											currentTag = getParser().next();
											geometry = GMLUtilsParser.parseGeometry(getParser(),attName,getFactory());
										}
										//Else, it does nothing									
									}
								}
								/****************
								 *  <COMPLEX>	*
								 ****************/
								else if ((gmlGeometry.isGML()==false)&&(attribute.getEntityType().getType() == IXMLType.COMPLEX)){
									if (attribute.isMultiple()){
										ArrayList multiple = null;
										if (values.get(attName) == null){
											multiple = new ArrayList();
										}else{
											multiple = (ArrayList)values.get(attribute.getName());
										}
										LinkedHashMap myValues = new LinkedHashMap(); 	
										geometry=parseComplexFeature(attribute,myValues,geometry);
										multiple.add(myValues);
										values.put(attName,multiple);
									}else{
										LinkedHashMap myValues = new LinkedHashMap(); 	
										geometry=parseComplexFeature(attribute,myValues,geometry);
										values.put(attName,myValues);
									}
								}
							}
						}	
					}
					break;
				case KXmlParser.END_TAG:
					if ((getParser().getName().compareTo(entity.getName()) == 0)){
						end = true;
					}
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if ((!end)&&(goNextTag==true)){				
					currentTag = getParser().next();
				}	
				else{
					goNextTag = true;
				}
			}
		}catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLParserException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new GMLFileReadException(e);
		} 	
		return geometry;
	}
	
	/**
	 * Parses a Complex type without Schema
	 * @param params
	 * @param values
	 * @return
	 * The geom
	 * @throws GMLException 
	 */
	private Object parseComplexFeatureNoSchema(String entityTag,Map values) throws BaseException{
		int currentTag;
		boolean end = false;
		Object geometry = null;

		try{		
			currentTag = getParser().nextTag();
			while (!end){	
				switch(currentTag){
				case KXmlParser.START_TAG:
					if (getParser().getName().compareTo(GMLTags.GML_BOUNDEDBY)==0){
						GMLUtilsParser.parseBoundedBy(getParser());
					}else{
						String namespace = getParser().getNameSpace();
						String tagName = getParser().getName();
						GMLGeometries gmlGeometry = new GMLGeometries(tagName);
						//If the tag hasn't namesapce it's local, maybe is an object declaration (or table component)
						
						/**************
						 * <GEOMETRY> *
						 **************/
						if ((namespace.compareTo("gml") == 0)||(gmlGeometry.isGML())){
							//if is GML geometry we try to parse it
							if (gmlGeometry.isGeometryGML()==true){
								geometry = GMLUtilsParser.parseGeometry(getParser(),tagName,getFactory());
							}
							else{
								if (gmlGeometry.isFeatureGML()==false){
									warnings.setElement(new GMLWarningWrongNamespace("gml"));
									//MAYBE IT HAD SCHEMA AND IS A GEOM DECLARED THERE, WE CAN TAKE IT LIKE THE IDENTIFIER OF THE GEOM
								}
								//Else it do nothing
							}
						}
						/**********
						 * <DATA> *
						 **********/
						else{
							//If another namespace it should be tables data  
							//it checks that it isn't geometry hide in other namespace
							if (gmlGeometry.isGML()==false){
								currentTag = getParser().next();
								String nextTagName = getParser().getName();
								gmlGeometry = new GMLGeometries(nextTagName);
								/****************
								 *  <SIMPLE>	*
								 ****************/
								if (currentTag==KXmlParser.TEXT){
									//it gets the value of the param, 
									//it doesn't know what type is, it takes string to default.
									String texto = getParser().getText();
									values.put(tagName,texto);
									
								}
								/****************
								 *  <COMPLEX>	*
								 ****************/
								else 
								{
									goNextTag = false;
									LinkedHashMap myValues = new LinkedHashMap(); 	
									parseComplexFeatureNoSchema("COMPLEX FEATURE",myValues);
									values.put(tagName,myValues);
								}
							}
						}	
					}
					break;
				case KXmlParser.END_TAG:
					if ((getParser().getName().compareTo(entityTag) == 0)||(getParser().getName().compareTo(getFeatureRoot()) == 0)){
						end = true;
					}
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if ((!end)&&(goNextTag==true)){				
					currentTag = getParser().next();
				}	
				else{
					goNextTag = true;
				}
			}
		}catch (XmlPullParserException e) {
			//Captured KXML parsing Exception
			throw new GMLParserException(e);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			throw new GMLFileReadException(e);
		} 	
		return geometry;	
	}

}
