/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.dataInput;

import java.awt.GridBagConstraints;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class DataInputContainer extends JPanel implements FocusListener, KeyListener{
	private static final long serialVersionUID = 7084105134015956663L;
	private boolean				decimal = false;
	private boolean				caracter = false;
	private JLabel lText = null;
	private JTextField jTextField = null;
	
	private String				focusText = null;

	/**
	 * This is the default constructor
	 */
	public DataInputContainer() {
		initialize();
	}

	/**
	 * This method initializes this
	 * @return void
	 */
	private void initialize() {
		GridBagConstraints gridBagConstraints1 = new java.awt.GridBagConstraints();
		GridBagConstraints gridBagConstraints2 = new java.awt.GridBagConstraints();

		gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 5);

		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints2.weightx = 1.0;

		setLayout(new java.awt.GridBagLayout());

		add(getLText(), gridBagConstraints1);
		add(getJTextField(), gridBagConstraints2);

		getJTextField().addFocusListener(this);
		getJTextField().addKeyListener(this);
		}

	private JLabel getLText(){
		if (lText == null){
			lText = new JLabel();
			lText.setText("JLabel");
			lText.setComponentOrientation(java.awt.ComponentOrientation.RIGHT_TO_LEFT);
		}
		return lText;
	}
	
	
	/**
	 * Da nombre al campo de texto del componente.
	 * @param text
	 */
	public void setLabelText(String text){
		this.lText.setText(text + ": ");
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getJTextField() {
		if (jTextField == null) {
			jTextField = new JTextField();
			jTextField.setBackground(java.awt.Color.white);
		}
		return jTextField;
	}
	
	/**
	 * Devuelve el valor del campo de texto del componente.
	 * @return
	 */
	public String getTextValue(){
		return this.jTextField.getText();
	}
	
	/**
	 * metodos para indicar y conocer el tipo de valor acepatado por el 
	 * campo de texto.
	 * @param flag
	 */
	public void setDecimal(boolean flag){
		this.decimal = flag;
	}
	
	public void setCaracter(boolean flag){
		this.caracter = flag; 
	}
	
	public boolean isDecimal(){
		return this.decimal;
	}
	
	public boolean isCaracter(){
		return this.caracter;
	}
	
	/**
	 * Habilita o deshabilita el control
	 * @param en
	 */
	public void setControlEnabled(boolean en){
		this.getJTextField().setEnabled(en);
		if (en)
			this.getJTextField().setBackground(java.awt.Color.white);
		else
			this.getJTextField().setBackground(getBackground());
	}
	
	/**
	 * Devuelve el valor del campo de texto.
	 * @return
	 */
	public String getValue(){
		return this.getJTextField().getText();
	}
	
	/**
	 * Asigna el valor al campo de texto.
	 * @return
	 */
	public void setValue(String value){
		this.getJTextField().setText(value);
	}
	
	
	/*********************************************************/
	/*********************LISTENERS***************************/
	/*********************************************************/
	
	
	public void focusGained(FocusEvent e) {
		if (e.getSource() == this.getJTextField()){
			if(!validateNumericFormat(this.getJTextField().getText()))
				this.getJTextField().setText("");
		}
		
	}

	public void focusLost(FocusEvent e) {
		//if(e.getSource() == this.getJTextField())
			//checkTextValue();
		
	}
	
	

	public void keyPressed(KeyEvent e) {
		
	}
		

	public void keyReleased(KeyEvent e) {
		String cad = this.getJTextField().getText();
		
		if (e.getSource()==this.getJTextField()){
			if (e.getKeyCode() == 10){
				if(!validateNumericFormat(cad))
					this.getJTextField().setText("");
				return;
			}
			if (!caracter){	
				if (checkTextField(e.getKeyCode()))
					focusText = cad;
				else
					this.getJTextField().setText(focusText);
			}
		}
	}

	public void keyTyped(KeyEvent e) {
		
	}
	
	
	/**
	 * Checkea un textField para que contenga caracteres validos. Consideramos validos
	 * los caracteres del 0 al 9, el punto, DEL, Supr, flechas y Enter teniendo en cuenta que el 
	 * cdigo ascii del teclado numerico es distinto.
	 */
	public boolean checkTextField(int code){
		
		//Si el caracter es erroneo pasamos de el y reponemos el texto que habia
		if(	(code >= 0 && code <= 7) || 
			code == 9 || 
			(code >= 11 && code <= 36) ||
			(code >= 41 && code <= 45) ||
			code == 47 ||
			(code >= 58 && code <= 95) ||
			(code >= 106 && code <= 109) ||
			(code >= 111 && code <= 126) ||
			code >= 128){
			return false;
		}
		
		return true;
	}
	
	/**
	 * Validacin de que el parmetro tenga formato numerico
	 * @param t Valor del campo
	 * @return true si es valido y false si no lo es
	 */
	private boolean validateNumericFormat(String field){
		try{
			new Double(field).doubleValue();
		}catch(NumberFormatException exc){
			return false;
		}
		return true;
	}
	
	
}
