/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.graphic;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.gvsig.gui.beans.BaseComponent;
import org.gvsig.gui.beans.graphic.listeners.GraphicListener;

/**
 * Control para el manejo de un grfico.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class GraphicContainer extends BaseComponent{
	private static final int		INTERNAL_MARGIN = 4;
	private static int				HEIGHT_CONTROL = 63;
	private int						HEIGHT_DOUBLESLIDER = 25;
	private static final int		HEIGHT_BOXES = 30;
	
	private int 					width = 400, height = 230;
	private JPanel 					jPanel = null;
	private JPanel 					jPanel2 = null;
	private JPanel 					pGeneral = null;
	private GraphicChartPanel 		pGraphic = null;
	private JPanel 					pDoubleSlider = null;
	private BoxesPanel 				pBoxes = null;
	private GraphicListener 		graphicListener = null;
	/**
	 * Variable que estar a true si se desea que se muestre el control de barra deslizadora
	 */
	private	boolean 				showSlider = true;

	
	/**
     * Constructor
     * @param width Ancho del grfico en pixeles
     * @param height Alto del grfico en pixeles
     */
	public GraphicContainer(int width, int height) {
		this.width = width;
		this.height = height;
		graphicListener = new GraphicListener(this);
		initialize();
	}
		
	/**
     * Constructor
     * @param width Ancho del grfico en pixeles
     * @param height Alto del grfico en pixeles
     */
	public GraphicContainer(int width, int height, boolean showSlider) {
		this.showSlider = showSlider;
		if(!showSlider)
			HEIGHT_DOUBLESLIDER = 0;
		this.width = width;
		this.height = height;
		graphicListener = new GraphicListener(this);
		initialize();
	}

	private void initialize() {
		FlowLayout layout = new FlowLayout();
		layout.setHgap(0);
		layout.setVgap(0);
        this.setLayout(layout);
        this.add(getPGeneral(), null);			
	}
	
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridy = 2;
			gridBagConstraints4.insets = new java.awt.Insets(2,0,0,0);
			gridBagConstraints4.gridwidth = 0;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.insets = new java.awt.Insets(2,0,0,0);
			gridBagConstraints3.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridy = 0;
			gridBagConstraints.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints.gridx = 0;
			pGeneral = new JPanel();
			//pGeneral.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.add(getPGraphic(), gridBagConstraints);
			if(showSlider)
				pGeneral.add(getPDoubleSlider(), gridBagConstraints3);
			pGeneral.add(getPBoxes(), gridBagConstraints4);
		}
		return pGeneral;
	}


	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public GraphicChartPanel getPGraphic() {
		if (pGraphic == null) {
			pGraphic = new GraphicChartPanel(width - INTERNAL_MARGIN, height - HEIGHT_BOXES - HEIGHT_DOUBLESLIDER);
		}
		return pGraphic;
	}


	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPDoubleSlider() {
		if (pDoubleSlider == null) {
			pDoubleSlider = new DoubleSliderControlPanel(width - INTERNAL_MARGIN, HEIGHT_DOUBLESLIDER, graphicListener);
		}
		return pDoubleSlider;
	}


	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private BoxesPanel getPBoxes() {
		if (pBoxes == null) {
			pBoxes = new BoxesPanel(width - INTERNAL_MARGIN, HEIGHT_BOXES);
		}
		return pBoxes;
	}
	
	/**
	 * Asigna el tamao del componente
	 * @param w Nuevo ancho del componente padre
	 * @param h Nuevo alto del componente padre
	 */
	public void setComponentSize(int w, int h){
		width = w;
		height = h;
        setPreferredSize(new java.awt.Dimension(w, h));
        //getPGeneral().setPreferredSize(new java.awt.Dimension(w, h));
		getPGraphic().setComponentSize(w - INTERNAL_MARGIN, h - HEIGHT_BOXES - HEIGHT_DOUBLESLIDER);
		if(this.showSlider){
			//getPDoubleSlider().setSize(w - INTERNAL_MARGIN, HEIGHT_DOUBLESLIDER);
			getPDoubleSlider().setPreferredSize(new java.awt.Dimension(w - INTERNAL_MARGIN, HEIGHT_DOUBLESLIDER));
		}
		((BoxesPanel)getPBoxes()).setComponentSize(w - INTERNAL_MARGIN, HEIGHT_BOXES);
	}
	
	//****************************************************
	//MTODOS DEL CONTROL

	/**
	 * Obtiene el valor de los controles. 
	 * @return Array con los valores de ambos controles. El primer valor del array es el control de la derecha
	 * y el segundo el de la izquierda.  
	 */
	public double[] getBoxesValues(){
		return getPBoxes().getBoxesValues();
	}
	
	/**
	 * Obtiene el botn de incremento del control izquierdo
	 * @return JButton. Botn de incremento del control izquierdo
	 */
	public JButton getPlusButtonControlLeft(){
		return this.getPBoxes().getControlLeft().getBmas();
	}
	
	/**
	 * Obtiene el botn de decremento del control izquierdo
	 * @return JButton. Botn de decremento del control izquierdo
	 */
	public JButton getLessButtonControlLeft(){
		return this.getPBoxes().getControlLeft().getBmenos();
	}
	
	/**
	 * Obtiene el botn de incremento del control derecho
	 * @return JButton. Botn de incremento del control derecho
	 */
	public JButton getPlusButtonControlRight(){
		return this.getPBoxes().getControlRight().getBmas();
	}
	
	/**
	 * Obtiene el botn de decremento del control derecho
	 * @return JButton. Botn de decremento del control derecho
	 */
	public JButton getLessButtonControlRight(){
		return this.getPBoxes().getControlRight().getBmenos();
	}
	
	/**
	 * Obtiene el JTextField de control derecho
	 * @return JTextField de control derecho
	 */
	public JTextField getTextControlRight(){
		return this.getPBoxes().getControlRight().getTText();
	}
	
	/**
	 * Obtiene el JTextField de control izquierdo
	 * @return JTextField de control izquierdo
	 */
	public JTextField getTextControlLeft(){
		return this.getPBoxes().getControlLeft().getTText();
	}
	
}
