/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.slidertext;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;

import org.gvsig.gui.beans.BaseComponent;
import org.gvsig.gui.beans.slidertext.listeners.SliderTextListener;

/**
 * Barra de deslizamiento con una ventana de texto que tiene el valor de la posicin de
 * la barra. En este control podr controlarse mediante la entrada de datos por la caja de
 * texto la posibilidad de introducir valores decimales.
 *  
 * Nacho Brodin (brodin_ign@gva.es)
 */

public class SliderTextContainer extends BaseComponent{

	private int 	MARGIN = 8;
	private int 	WTEXT = 50;
	private JPanel pSlider = null;
	private JPanel pText = null;
	private JSlider slider = null;
	private JTextField text = null;
	private int	width = 260;
	private int	height = 35;
	private int min = 0;
	private int max = 255;
	private int defaultPos = 0;
	private SliderTextListener listener = null;
	
	/**
	 * Contructor
	 * @param width Ancho del componente
	 * @param height Alto del componente
	 * @param min Valor mnimo de la barra
	 * @param max Valor mximo de la barra
	 * @param defaultPos Posicin por defecto 
	 */
	public SliderTextContainer(int width, int height, int min, int max, int defaultPos) {
		super();
		this.width = width;
		this.height = height;
		this.min = min;
		this.max = max;
		this.defaultPos = defaultPos;
	
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		listener = new SliderTextListener(this);
        GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
        gridBagConstraints1.gridx = 1;
        gridBagConstraints1.gridy = 0;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        this.setLayout(new GridBagLayout());
        this.add(getPSlider(), gridBagConstraints);
        this.add(getPText(), gridBagConstraints1);
        this.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
        setComponentSize(width, height);
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPSlider() {
		if (pSlider == null) {
			pSlider = new JPanel();
			pSlider.add(getSlider(), null);
		}
		return pSlider;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPText() {
		if (pText == null) {
			pText = new JPanel();
			pText.setPreferredSize(new java.awt.Dimension(WTEXT,35));
			pText.add(getText(), null);
		}
		return pText;
	}

	/**
	 * This method initializes jSlider	
	 * 	
	 * @return javax.swing.JSlider	
	 */
	public JSlider getSlider() {
		if (slider == null) {
			slider = new JSlider();
			slider.setMajorTickSpacing(50);
			slider.setPaintTicks(true);
			slider.setMinorTickSpacing(10);
			slider.setMinimum(min);
			slider.setMaximum(max);
			slider.setValue(defaultPos);
			slider.addChangeListener(listener);
		}
		return slider;
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getText() {
		if (text == null) {
			text = new JTextField();
			text.setText(defaultPos+"");
			text.addFocusListener(listener);
			text.addKeyListener(listener);
		}
		return text;
	}
	
	public void setComponentSize(int w, int h){
		this.width = w;
		this.height = h;
        this.setPreferredSize(new java.awt.Dimension(width, height));
		pSlider.setPreferredSize(new java.awt.Dimension(width - MARGIN - WTEXT, 27));
		pText.setPreferredSize(new java.awt.Dimension(WTEXT,35));
		slider.setPreferredSize(new java.awt.Dimension(width - WTEXT - MARGIN,27));
		text.setPreferredSize(new java.awt.Dimension(WTEXT - 10, 19));
	}
	
	/**
	 * Obtiene el valor del control.
	 * @return Valor del control en formato double.
	 */
	public double getValue(){
		return Double.valueOf(getText().getText()).doubleValue();
	}

	/**
	 * Asigna el valor del control.
	 * @return Valor del control en formato double.
	 */
	public void setValue(double value){
		getText().setText(String.valueOf(value));
		getSlider().setValue((int)value);
	}
}
