/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.table;

import java.awt.FlowLayout;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.gvsig.i18n.Messages;

/**
 * Control para el manejo de tablas. No contiene eventos, estos deben 
 * ser manejados desde la clase que lo llame.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class TableControlerPanel extends JPanel{
		
	//**********************Vars**********************************
	private JButton 					bPrev = null;
	private JComboBox 					cPoint = null;
	private JButton 					bNext = null;
	private JButton 					bFirst = null;
	private JButton 					bLast = null;
	private JPanel 						pTableControl = null;
	private JLabel 						lPoints = null;
	private JLabel 						lNumberOfPoints = null;
	private JButton 					bNew = null;
	private JButton 					bDelPoint = null;
	private JButton 					bClear = null;
	private String						pathToImages = "images/";//"/com/iver/cit/gvsig/gui/panels/images/";
	/**
	 * Estado de los botones cuando se ejecuta disableAllControls
	 */
	private boolean[] 					buttonsState = new boolean[8];
	/**
	 * DisableAllControls ha sido ejecutada si est a true esta variabled
	 */
	private boolean 					disableAllControls = false;
	//**********************End Vars******************************
	
	//**********************Methods*******************************
	/**
	 * This is the default constructor
	 */
	public TableControlerPanel(ActionListener buttonsListener) {
		super();
		initialize(buttonsListener);

	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize(ActionListener buttonsListener) {                
		this.setPreferredSize(new java.awt.Dimension(295,22));
		this.setSize(new java.awt.Dimension(295,22));
		FlowLayout flowLayout5 = new FlowLayout();
		flowLayout5.setHgap(0);
		flowLayout5.setVgap(0);
        
		this.setLayout(flowLayout5);
		
		lPoints = new JLabel();
		lPoints.setText(Messages.getText("puntos"));
		lPoints.setPreferredSize(new java.awt.Dimension(46,15));
		this.add(lPoints, null);
		
		this.add(getBFirst(), null);
		this.add(getBPrev(), null);
		this.add(getCPoint(), null);
		this.add(getBNext(), null);
		this.add(getBLast(), null);
		this.add(getBNew(), null);
		this.add(getLNumberOfPoints(), null);
		this.add(getBDelPoint(), null);
		this.add(getBClear(), null);
		
		getBFirst().addActionListener(buttonsListener);
		getBPrev().addActionListener(buttonsListener);
		getCPoint().addActionListener(buttonsListener);
		getBNext().addActionListener(buttonsListener);
		getBLast().addActionListener(buttonsListener);
		getBNew().addActionListener(buttonsListener);
		getBDelPoint().addActionListener(buttonsListener);
		getBClear().addActionListener(buttonsListener);
	}
	
	/**
	 * Esta funcin deshabilita todos los controles y guarda sus valores
	 * de habilitado o deshabilitado para que cuando se ejecute restoreControlsValue
	 * se vuelvan a quedar como estaba
	 */
	public void disableAllControls(){
		if(!disableAllControls){
			disableAllControls = true;
			//Salvamos los estados
			
			buttonsState[0] = getBFirst().isEnabled();
			buttonsState[1] = getBPrev().isEnabled();
			buttonsState[2] = getCPoint().isEnabled();
			buttonsState[3] = getBNext().isEnabled();
			buttonsState[4] = getBLast().isEnabled();
			buttonsState[5] = getBDelPoint().isEnabled();
			buttonsState[6] = getBClear().isEnabled();
			buttonsState[7] = getBNew().isEnabled();
						
			//Desactivamos controles
			getBFirst().setEnabled(false);
			getBPrev().setEnabled(false);
			getCPoint().setEnabled(false);
			getBNext().setEnabled(false);
			getBLast().setEnabled(false);
			getBDelPoint().setEnabled(false);
			getBClear().setEnabled(false);
			getBNew().setEnabled(false);
		}
	}
	
	/**
	 * Esta funcin deja los controles como estaban al ejecutar la funcin 
	 * disableAllControls
	 */
	public void restoreControlsValue(){
		if(disableAllControls){
			disableAllControls = false;
			getBFirst().setEnabled(buttonsState[0]);
			getBPrev().setEnabled(buttonsState[1]);
			getCPoint().setEnabled(buttonsState[2]);
			getBNext().setEnabled(buttonsState[3]);
			getBLast().setEnabled(buttonsState[4]);
			getBDelPoint().setEnabled(buttonsState[5]);
			getBClear().setEnabled(buttonsState[6]);
			getBNew().setEnabled(buttonsState[6]);
		}
	}
	//**********************Methods*******************************
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBFirst() {
		if (bFirst == null) {
			bFirst = new JButton();
			bFirst.setPreferredSize(new java.awt.Dimension(22,22));
			bFirst.setEnabled(false);
			bFirst.setIcon(new ImageIcon(getClass().getResource(pathToImages + "first.png")));
			bFirst.setToolTipText(Messages.getText("primero"));
		}
		return bFirst;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBLast() {
		if (bLast == null) {
			bLast = new JButton();
			bLast.setPreferredSize(new java.awt.Dimension(22,22));
			bLast.setEnabled(false);
			bLast.setIcon(new ImageIcon(getClass().getResource(pathToImages + "last.png")));
			bLast.setToolTipText(Messages.getText("ultimo"));
		}
		return bLast;
	}
	
	/**
	 * This method initializes bBefore	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBPrev() {
		if (bPrev == null) {
			bPrev = new JButton();
			bPrev.setText("");
			bPrev.setEnabled(false);
			bPrev.setPreferredSize(new java.awt.Dimension(22,22));
			bPrev.setIcon(new ImageIcon(getClass().getResource(pathToImages + "prev.png")));
			bPrev.setActionCommand("");
			bPrev.setToolTipText(Messages.getText("anterior"));
		}
		return bPrev;
	}

	/**
	 * This method initializes bNext	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBNext() {
		if (bNext == null) {
			bNext = new JButton();
			bNext.setText("");
			bNext.setEnabled(false);
			bNext.setPreferredSize(new java.awt.Dimension(22,22));
			bNext.setIcon(new ImageIcon(getClass().getResource(pathToImages + "next.png")));
			bNext.setActionCommand("");
			bNext.setToolTipText(Messages.getText("siguiente"));
		}
		return bNext;
	}
	
	/**
	 * Este mtodo inicializa el combo que contiene el nmero de puntos.	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	public JComboBox getCPoint() {
		if (cPoint == null) {
			cPoint = new JComboBox();
			cPoint.setPreferredSize(new java.awt.Dimension(50,22));
			
		}
		return cPoint;
	}

	/**
	 * @return Returns the lNumberOfPoints.
	 */
	public JLabel getLNumberOfPoints() {
		if(lNumberOfPoints == null){
			lNumberOfPoints = new JLabel();
			lNumberOfPoints.setText(Messages.getText("de")+" 0");
			lNumberOfPoints.setPreferredSize(new java.awt.Dimension(35,15));
		}
		return lNumberOfPoints;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBNew() {
		if (bNew == null) {
			bNew = new JButton();
			bNew.setPreferredSize(new java.awt.Dimension(22,22));
			bNew.setIcon(new ImageIcon(getClass().getResource(pathToImages + "newpoint.png")));
			bNew.setToolTipText(Messages.getText("nuevo"));
		}
		return bNew;
	}
	
	/**
	 * Este mtodo inicializa el botn del eliminar punto que har que se elimine
	 * el punto seleccionado.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBDelPoint() {
		if (bDelPoint == null) {
			bDelPoint = new JButton();
			bDelPoint.setText("");
			bDelPoint.setEnabled(false);
			bDelPoint.setPreferredSize(new java.awt.Dimension(22,22));
			bDelPoint.setIcon(new ImageIcon(getClass().getResource(pathToImages + "delone.png")));
			bDelPoint.setToolTipText(Messages.getText("borrar_uno"));
		}
		return bDelPoint;
	}
	
	/**
	 * Este mtodo inicializa el botn del clear que har que se eliminen todos los
	 * puntos seleccionados.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBClear() {
		if (bClear == null) {
			bClear = new JButton();
			bClear.setText("");
			bClear.setPreferredSize(new java.awt.Dimension(22,22));
			bClear.setEnabled(false);
			bClear.setIcon(new ImageIcon(getClass().getResource(pathToImages + "delall.png")));
			bClear.setActionCommand("");
			bClear.setToolTipText(Messages.getText("borrar_todos"));
		}
		return bClear;
	}
	
	/**
	 * Resetea el control al estado inicial. Limpia el combo, pone
	 * el nmero de elementos a 0 y desactiva las flechas.
	 *
	 */
	public void resetControls(){
		getCPoint().removeAllItems();
		getLNumberOfPoints().setText(Messages.getText("de 0"));
		checkArrows();
	}
	
	/**
	 * Elimina del control un elemento de una posicin.
	 * <UL>
	 * <LI>Actualiza el combo</LI>
	 * <LI>Actualiza el texto que dice el nmero de elementos</LI>
	 * <LI>Actualiza las flechas</LI>
	 * </UL>
	 * @param pos Posicin del elemento a eliminar.
	 */
	public void setNItems(int n){
		//if(n < 1)
			//return;
		getCPoint().removeAllItems();
		for(int i=1;i<=n;i++){
			try{
				getCPoint().addItem(String.valueOf(i));
			}catch(ArrayIndexOutOfBoundsException ex){
				//No aadimos nada
			}
		}
		getLNumberOfPoints().setText(Messages.getText("de ") + " " + n);
		checkArrows();
	}
	
	/**
	 * Selecciona un elemento del control
	 * @param index
	 */
	public void setSelectedIndex(int index){
		try{
			getCPoint().setSelectedIndex(index);
		}catch(IllegalArgumentException ex){
			
		}
		checkArrows();
	}
	
	/**
	 * Devuelve el punto seleccionado
	 * @return Punto seleccionado.
	 */
	public int getSelectedIndex(){
		return getCPoint().getSelectedIndex();
	}
	
	/**
	 * Obtiene el nmero de elementos en la lista.
	 * @return Nmero de elementos
	 */
	public int getItemCount(){
		return getCPoint().getItemCount();
	}
	
	/**
	 * Obtiene el Objeto seleccionado como cadena de texto.
	 * @return Nmero seleccionado
	 */
	public String getSelectedItem(){
		return getCPoint().getSelectedItem().toString();
	}
	
	/**
	 * Comprueba la posicin del combo para ver si tiene que
	 * habilitar o deshabilitar las flechas de delante y detrs.
	 */
	public void checkArrows(){
		if(disableAllControls)
			return;
		
		if(getCPoint().getItemCount() <= 0){
			getBClear().setEnabled(false);
			getBDelPoint().setEnabled(false);
		}else{
			getBClear().setEnabled(true);
			getBDelPoint().setEnabled(true);
		}
		
		if(getCPoint().getSelectedIndex() == -1){
			getBPrev().setEnabled(false);
			getBNext().setEnabled(false);
			getBLast().setEnabled(false);
			getBFirst().setEnabled(false);
			
			return;
		}
		if(getCPoint().getSelectedIndex() == 0){
			getBPrev().setEnabled(false);
			getBFirst().setEnabled(false);
		}else{
			getBPrev().setEnabled(true);
			getBFirst().setEnabled(true);
		}
		
		if(getCPoint().getSelectedIndex() == (getCPoint().getItemCount() - 1) ){
			getBNext().setEnabled(false);
			getBLast().setEnabled(false);
		}else{
			getBNext().setEnabled(true);
			getBLast().setEnabled(true);
		}
	}
	
	/**
	 * Aade un punto al combo y checkea los controles colocandolos en los valores correctos.
	 * @param countPoints Nmero de punto a aadir al final del combo
	 */
	public void addPointToTable(int countPoints){
		getCPoint().addItem("" + countPoints);
		getLNumberOfPoints().setText(Messages.getText("de ") + " " + countPoints);
		setSelectedIndex(getItemCount() - 1);
		checkArrows();
	}
	
	/**
	 * Asigna la ruta donde estn las imagenes
	 * @param pathToImages
	 */
	public void setPathToImages(String pathToImages) {
		this.pathToImages = pathToImages;
	}
	
	/**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
	    this.getBDelPoint().setEnabled(enabled);
	    this.getBClear().setEnabled(enabled);
    }

    /**
     * Dice si los controles estn deshabilitados o no
     * @return true si est habilitado y false si no lo est
     */
	public boolean isDisableAllControls() {
		return disableAllControls;
	}
 }
