/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*/
package org.gvsig.gui.beans.table.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import org.gvsig.gui.beans.table.Table;
import org.gvsig.gui.beans.table.TableContainer;
import org.gvsig.gui.beans.table.TableControlerPanel;
import org.gvsig.gui.beans.table.exceptions.NotInitializeException;
import org.gvsig.gui.beans.table.models.IModel;

/**
 * Listener para la tabla 
 * @author Nacho Brodin <brodin_ign@gva.es>
 *
 */
public class TableListener implements TableModelListener, ActionListener{

	public static boolean 			comboEventEnable = true;
	private TableContainer		 	tableContainer = null;
	private TableControlerPanel 	controlPanel = null;
	private Table 					table= null;
	public boolean					enableNewLineListener = true;
	
	/**
	 * Constructor
	 * @param tableContainer
	 */
	public TableListener(TableContainer tableContainer){
		this.tableContainer = tableContainer;
	}
	
	private void initialize(){
		this.controlPanel = tableContainer.getPTableControl();
		this.table = tableContainer.getPTable();
	}
	
	/**
	 * Captura y gestiona los eventos del control de la tabla para aadir filas, 
	 * eliminar filas y moverse por la tabla.
	 */
	public void actionPerformed(ActionEvent e) {
		if(controlPanel == null || table == null)
			initialize();
		try{
			/**
			 * Nueva entrada
			 */
			if(enableNewLineListener && e.getSource() == controlPanel.getBNew()){
				if(table.getTableModel() instanceof IModel)
					tableContainer.addRow(((IModel)table.getTableModel()).getNewLine());
			}
			
			/**
			 * Eliminar todas las filas
			 */
			if(enableNewLineListener && e.getSource() == controlPanel.getBClear()){
				tableContainer.removeAllRows();
			}
			
			/**
			 * Elimina una entrada concreta de la tabla
			 */
			if(enableNewLineListener && e.getSource() == controlPanel.getBDelPoint()){
				tableContainer.delRow(tableContainer.getSelectedIndex());
			}
			
			if(e.getSource() == controlPanel.getBFirst()){
				tableContainer.setSelectedIndex(0);
			}
			
			if(e.getSource() == controlPanel.getBLast()){
				tableContainer.setSelectedIndex(tableContainer.getRowCount() - 1);
			}
			
			if(e.getSource() == controlPanel.getBNext()){
				if(tableContainer.getSelectedIndex() < tableContainer.getRowCount())
					tableContainer.setSelectedIndex(tableContainer.getSelectedIndex() + 1);
			}
			
			if(e.getSource() == controlPanel.getBPrev()){
				if(tableContainer.getSelectedIndex() > 0)
					tableContainer.setSelectedIndex(tableContainer.getSelectedIndex() - 1);
			}
	
			if(e.getSource() == controlPanel.getCPoint()){
				if(comboEventEnable)
					tableContainer.setSelectedIndex(controlPanel.getCPoint().getSelectedIndex());
			}
			
		}catch(NotInitializeException ex){
			
		}
	}

	public void tableChanged(TableModelEvent e) {
		// TODO Auto-generated method stub
		
	}

}
