/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.gui.beans.textincreaser;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.gvsig.gui.beans.BaseComponent;

/**
 * 
 * Nacho Brodin (brodin_ign@gva.es)
 */

public class TextIncreaserContainer extends BaseComponent implements ActionListener{
	
	private int 				width = 70, height = 25;
	private JTextField 			tText = null;
	private JPanel				pButtons = null;
	private JButton 			bmas = null;
	private JButton 			bmenos = null;
	private JPanel				pGeneral = null;
	private	double 				minValue = 0;
	private	double 				maxValue = 100;
	private	double 				init = 0;
	private String				pathToImages = "images/";//"/com/iver/cit/gvsig/gui/panels/images/";
	/**
	 * Variable que est a true si los controles estn a la derecha
	 */
	private boolean isRight = true;
	
	/**
	 * Creacin de un componente TextIncrearserContainer
	 * @param width Ancho del componente
	 * @param minValue Mnimo valor que puede tomar el texto del control
	 * @param maxValue Mximo valor que puede tomar el texto del control
	 * @param right	a true si queremos que los controles se muestren a la derecha del TextBox
	 * y false si los queremos a la izquierda
	 */
	public TextIncreaserContainer(int width, double minValue, double maxValue, double init, boolean right) {
		super();
		this.width = width;
		this.minValue = minValue;
		this.maxValue = maxValue;
		this.isRight = right;
		this.init = init;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        FlowLayout flowLayout = new FlowLayout();
        flowLayout.setHgap(0);
        flowLayout.setVgap(0);
        FlowLayout flowLayout5 = new FlowLayout();
        flowLayout5.setHgap(0);
        flowLayout5.setVgap(0);
        this.setLayout(flowLayout5);
        //this.setBorder(javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray,1));
        this.setSize(new java.awt.Dimension(width, width));
        this.setPreferredSize(new java.awt.Dimension(width, width));
        this.add(getPGeneral(), null);
			
	}

	/**
	 * Asigna un valor al textBox
	 * @param value
	 */
	public void setValue(double value){
		getTText().setText(Double.toString(value));
	}
	
	/**
	 * Obtiene el valor del textBox
	 * @return value
	 */
	public double getValue(){
		try{
			Double.valueOf(getTText().getText()).doubleValue();	
		}catch(NumberFormatException exc){
			getTText().setText("0.0");
			return 0;
		}
		return Double.valueOf(getTText().getText()).doubleValue();
	}
	
	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getTText() {
		if (tText == null) {
			tText = new JTextField();
			tText.setText(Double.toString(init));
			tText.setPreferredSize(new java.awt.Dimension(width - 25,25));
		}
		return tText;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPButtons() {
		if (pButtons == null) {
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints3.gridy = 1;
			gridBagConstraints3.gridx = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints2.gridy = 0;
			gridBagConstraints2.gridx = 0;
			pButtons = new JPanel();
			pButtons.setLayout(new GridBagLayout());
			pButtons.setPreferredSize(new java.awt.Dimension(25,25));
			pButtons.add(getBmas(), gridBagConstraints2);
			pButtons.add(getBmenos(), gridBagConstraints3);
		}
		return pButtons;
	}

	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBmas() {
		if (bmas == null) {
			bmas = new JButton();
			bmas.setPreferredSize(new java.awt.Dimension(25,12));
			bmas.addActionListener(this);
			bmas.setIcon(new ImageIcon(getClass().getResource(pathToImages + "mas.png")));
		}
		return bmas;
	}

	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBmenos() {
		if (bmenos == null) {
			bmenos = new JButton();
			bmenos.setPreferredSize(new java.awt.Dimension(25,12));
			bmenos.addActionListener(this);
			bmenos.setIcon(new ImageIcon(getClass().getResource(pathToImages + "menos.png")));
		}
		return bmenos;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPGeneral() {
		if (pGeneral == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();

			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
			if(isRight){
				gridBagConstraints1.gridx = 1;
				gridBagConstraints.gridx = 0;
			}else{
				gridBagConstraints1.gridx = 0;
				gridBagConstraints.gridx = 1;
			}
			gridBagConstraints.gridy = 0;
			gridBagConstraints1.gridy = 0;
			gridBagConstraints.weightx = 1.0;
			pGeneral = new JPanel();
			pGeneral.setLayout(new GridBagLayout());
			pGeneral.setPreferredSize(new java.awt.Dimension(width, 25));
			pGeneral.add(getTText(), gridBagConstraints);
			pGeneral.add(getPButtons(), gridBagConstraints1);
		}
		return pGeneral;
	}

	public void actionPerformed(ActionEvent e) {
		double value = Double.parseDouble(getTText().getText());
		if(e.getSource() == bmas){
			value ++;
			if(value <= maxValue)
				tText.setText(Double.toString(value));
		}
		
		if(e.getSource() == bmenos){
			value --;
			if(value >= minValue)
				tText.setText(Double.toString(value));
		}
	}

}
